' ********************************************************************************
'     Document    :  frmAudioDropBox.vb
'     Description :  A small form for adding audio tracks to the compilation
' ********************************************************************************

Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports System.IO
Imports CDBurnerXP.IO
Imports Microsoft.Win32
Imports CDBurnerXP.Controls
Imports CDBurnerXP.Drawing
Imports RocketDivision
Imports CDBurnerXP

Public Class frmAudioDropBox
  Private m_OwnerForm As frmAudioCompilation = frmAudioCompilation.AudioInstance
  Private m_LayoutManager As AudioLayoutManager
  Private m_Device As StarBurnX.Drive
  Private m_Track As String
  Private m_OnDisc As String
  Private m_Total As String
  Private m_Free As String
  Private m_FreeTwo As String
  Private m_MoveMouseDown As Boolean
  Private m_MoveX As Integer
  Private m_MoveY As Integer

  <CLSCompliant(False)> _
  Public Sub New(ByRef mDevice As StarBurnX.Drive, _
                                       ByRef Owner As frmAudioCompilation, _
                                       ByRef layoutManager As AudioLayoutManager)

    ' This call is required by the Windows Form Designer.
    InitializeComponent()

    m_Device = mDevice
    m_LayoutManager = layoutManager
    m_OwnerForm = Owner
    UpdateSizeBarValue()
  End Sub

  Private Sub frmAudioDropBox_Load(ByVal sender As Object, _
                                   ByVal e As System.EventArgs) _
          Handles MyBase.Load
    SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint Or ControlStyles.AllPaintingInWmPaint, True)
    'Language Set
    m_Track = RM.GetString("Tracks: {0}")
    m_OnDisc = RM.GetString("Used: {0}")
    m_Total = RM.GetString("Estimated size: {0}")
    m_Free = RM.GetString("Remaining: {0}")
    m_FreeTwo = RM.GetString("Remaining: (Unknown)")
    cmnuDropboxPaste.Text = RM.GetString("Paste")
    Sizebar.DiscType = CDBurnerXP.Controls.CDBSizebar.enmDiscType.AudioDisc
    Sizebar.UseWarning = False
    m_OwnerForm.MsgBoxOwner = Me
    'Use same location setting for both data and audio
    Me.Location = ApplicationSettings.DataDropFormLocation

    UpdateSizeBarValue()
    picBkg.Image = RtlHelper.FlipIfNecessary(picBkg.Image, Me.RightToLeftLayout)
  End Sub

  Protected Overrides Sub OnClosed(ByVal e As System.EventArgs)
    MyBase.OnClosed(e)

    m_OwnerForm.MsgBoxOwner = m_OwnerForm
    ApplicationSettings.DataDropFormLocation = Me.Location
  End Sub

  Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
    MyBase.OnPaint(e)
    e.Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias
    e.Graphics.DrawImage(picBkg.Image, CSng((Me.Width - picBkg.Image.Width) / 2), CSng(( _
            Me.Height - picBkg.Image.Height) / 2), picBkg.Image.Width, picBkg.Image.Height)

    If Me.RightToLeftLayout Then
      e.Graphics.DrawRectangle(Pens.Black, New Rectangle(Me.DisplayRectangle.X + 1, Me.DisplayRectangle.Y, Me.DisplayRectangle.Width - 1, Me.DisplayRectangle.Height - 1))
    Else
      e.Graphics.DrawRectangle(Pens.Black, New Rectangle(Me.DisplayRectangle.X, Me.DisplayRectangle.Y, Me.DisplayRectangle.Width - 1, Me.DisplayRectangle.Height - 1))
    End If

    Dim brush As New SolidBrush(Color.FromArgb(180, 0, 0, 0))
    'Draw Info Strings
    e.Graphics.DrawString(String.Format(m_Track, m_LayoutManager.Tracks.Length), Me.Font, brush, _
            4, 12)
    e.Graphics.DrawString(String.Format(m_OnDisc, FormatFileSize.Format(Sizebar.OnDiscValue)), Me.Font, brush, 4, 36)
    e.Graphics.DrawString(String.Format(m_Total, FormatFileSize.Format(Sizebar.OnDiscValue + Sizebar.BarValue)), Me.Font, brush, 4, 48)

    If Not m_Device Is Nothing AndAlso m_Device.DiscInfo.DiscType <> StarBurnX.STARBURN_DISC_TYPE.STARBURN_DISC_TYPE_NONE Then
      e.Graphics.DrawString(String.Format(m_Free, FormatFileSize.Format(Sizebar.OverDiscValue - _
              Sizebar.BarValue - Sizebar.OnDiscValue)), Me.Font, brush, 4, 60)

    Else
      e.Graphics.DrawString(m_FreeTwo, Me.Font, brush, 4, 60)
    End If

    brush.Dispose()
  End Sub

  Private Sub frmAudioDropBox_DragOver(ByVal sender As Object, _
                                       ByVal e As System.Windows.Forms.DragEventArgs) _
          Handles MyBase.DragOver
    e.Effect = DragDropEffects.Copy
  End Sub

  Private Sub UpdateSizeBarValue()
    Sizebar.MaxValue = m_OwnerForm.Sizebar3.MaxValue
    Sizebar.BarValue = m_OwnerForm.Sizebar3.BarValue
    Sizebar.OnDiscValue = m_OwnerForm.Sizebar3.OnDiscValue
    Sizebar.OverDiscValue = m_OwnerForm.Sizebar3.OverDiscValue
    Sizebar.UpdateSizeBar()
    Me.Invalidate()
  End Sub

  Private Sub frmAudioDropBox_DragDrop(ByVal sender As Object, _
                                       ByVal e As System.Windows.Forms.DragEventArgs) _
          Handles MyBase.DragDrop
    'Hard work here
    Me.Cursor = Cursors.WaitCursor

    If e.Data.GetDataPresent(DataFormats.FileDrop) Then
      m_OwnerForm.AddToCompilation(CType(e.Data.GetData(DataFormats.FileDrop), String()), -1)
    End If

    UpdateSizeBarValue()
    Me.Cursor = Cursors.Default
  End Sub

  Private Sub cmnuDropbox_Opening(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmnuDropbox.Popup

    If SafeClipboard.IsDataPresent(DataFormats.FileDrop) Then
      cmnuDropboxPaste.Enabled = True
    Else
      cmnuDropboxPaste.Enabled = False
    End If

  End Sub

  Private Sub frmAudioDropBox_MouseDown(ByVal sender As Object, _
                                        ByVal e As System.Windows.Forms.MouseEventArgs) _
          Handles MyBase.MouseDown

    If e.Button = System.Windows.Forms.MouseButtons.Left Then
      m_MoveMouseDown = True
      m_MoveX = e.X
      m_MoveY = e.Y
    End If

  End Sub

  Private Sub frmAudioDropBox_MouseUp(ByVal sender As Object, _
                                      ByVal e As System.Windows.Forms.MouseEventArgs) _
          Handles MyBase.MouseUp
    m_MoveMouseDown = False
  End Sub

  Private Sub frmAudioDropBox_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseMove
    If m_MoveMouseDown Then
      If Me.RightToLeftLayout Then
        Me.Left = Me.Left - (e.X - m_MoveX)
      Else
        Me.Left = Me.Left + (e.X - m_MoveX)
      End If
      Me.Top = Me.Top + (e.Y - m_MoveY)
    End If
  End Sub

  Private Sub mnuDropboxPaste_Click(ByVal sender As Object, _
                                    ByVal e As System.EventArgs) _
          Handles cmnuDropboxPaste.Click

    If SafeClipboard.IsDataPresent(DataFormats.FileDrop) Then
      m_OwnerForm.AddToCompilation(CType(SafeClipboard.GetData(DataFormats.FileDrop), String()), -1)
    End If

  End Sub

  Private Sub frmAudioDropBox_KeyDown(ByVal sender As Object, _
                                      ByVal e As System.Windows.Forms.KeyEventArgs) _
          Handles MyBase.KeyDown

    Select Case e.KeyCode

      Case Keys.V

        If e.Control = True Then

          If SafeClipboard.IsDataPresent(DataFormats.FileDrop) Then
            m_OwnerForm.AddToCompilation(CType(SafeClipboard.GetData(DataFormats.FileDrop), String()), -1)
          End If

        End If

      Case Else
        'Do nothing

    End Select

  End Sub

  Protected Overrides Sub OnMediaChanged(ByVal VolumeLabel As String)
    If Not m_Device Is Nothing Then
      m_Device.DiscInfo.Refresh()
      Me.Invalidate()
      UpdateSizeBarValue()
    End If
  End Sub


  Private Sub cmdCancel_Click(ByVal sender As System.Object, _
                              ByVal e As System.EventArgs) _
          Handles cmdCancel.Click
    Me.Close()
  End Sub

  Private Delegate Sub DragDropDelegate(ByVal Data() As String, ByVal insertindex As Integer)

  Private Sub tmrUpdateSizebar(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrUpdateBar.Tick
    UpdateSizeBarValue()
  End Sub

End Class
