Imports Bass = Un4seen.Bass.bass
Imports BassCD = Un4seen.Bass.AddOn.Cd.BassCd
Imports BassTags = Un4seen.Bass.AddOn.Tags.BassTags
Imports System.Globalization
Imports System.Resources
Imports System.IO
Imports CDBurnerXP.IO
Imports CDBurnerXP.Controls

Public Class frmAudioPlayer
  Private Stream As Integer
  Private m_bPlaying As Boolean = False
  Private m_bPaused As Boolean = False
  Private m_LastPlayed As String = String.Empty
  Private m_Playlist As New Stack(Of String)
  Private m_UpdatingUi As Boolean = False
  Private m_MyMenuItem As MenuItem

#Region "Properties"

  Public Property MyMenuItem() As MenuItem
    Get
      Return m_MyMenuItem
    End Get
    Set(ByVal value As MenuItem)
      m_MyMenuItem = value
    End Set
  End Property

#End Region

  Private Sub frmAudioPlayer_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
    PositionTimer.ElapsedText = RM.GetString("Elapsed")
    PositionTimer.RemainingText = RM.GetString("Remaining")
    Me.Text = RM.GetString("Audio Player")
  End Sub

#Region "Audio functions"

  Public Sub PlayAudio(ByVal items() As String)
    If items Is Nothing Then Exit Sub

    m_Playlist.Clear()
    For i As Integer = items.Length - 1 To 0 Step -1
      m_Playlist.Push(items(i))
    Next

    PlayAudio(m_Playlist.Pop())
  End Sub

  Public Sub PlayAudio(ByVal strsPath As String)
    If m_bPlaying Then
      StopAudio()
    End If

    ' Use last played file?
    If strsPath = String.Empty And m_LastPlayed <> String.Empty Then
      strsPath = m_LastPlayed
    End If

    ' Nothing to play?
    If strsPath = String.Empty Then Exit Sub

    m_LastPlayed = strsPath

    Stream = Bass.BASS_StreamCreateFile(strsPath, 0, 0, 0)
    If Stream = 0 Then
      MessageBox.Show(Me, String.Format(RM.GetString("Error opening file {0}"), Path.GetFileName(strsPath)), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1)
      Exit Sub
    End If

    Me.cmdPlay.Image = Global.CDBurnerXP_Pro.My.Resources.Resources.AudioPlayerPause

    If Playlists.IsCda(strsPath) Then
      TextScroller.Texts = Path.GetFileName(strsPath)
    Else
      Dim taginfo As Un4seen.Bass.AddOn.Tags.TAG_INFO
      taginfo = BassTags.BASS_TAG_GetFromFile(strsPath)

      Dim desc As String = AudioTrack.FormatAudioTags(taginfo)
      If String.IsNullOrEmpty(desc) Then
        TextScroller.Texts = Path.GetFileName(strsPath)
      Else
        TextScroller.Texts = desc
      End If
    End If

    TextScroller.StartScroller()
    PositionTimer.TotalTimeInSeconds = Convert.ToInt32(Bass.BASS_ChannelBytes2Seconds(Stream, Bass.BASS_ChannelGetLength(Stream)))
    If Stream <> 0 Then
      ' play the filestream
      Bass.BASS_ChannelPlay(Stream, True)
      tmrAudio.Enabled = True
    End If

    m_bPlaying = True
  End Sub

  Public Sub StopAudio()
    If Not m_bPlaying Then Exit Sub

    Me.cmdPlay.Image = Global.CDBurnerXP_Pro.My.Resources.Resources.AudioPlayerPlay

    Dim currentVolume As Single
    Bass.BASS_ChannelGetAttribute(Stream, Un4seen.Bass.BASSAttribute.BASS_ATTRIB_VOL, currentVolume)
    If Not m_bPaused Then
      If Not (Bass.BASS_ChannelIsActive(Stream) = Un4seen.Bass.BASSActive.BASS_ACTIVE_STOPPED) Then
        Bass.BASS_ChannelSlideAttribute(Stream, Un4seen.Bass.BASSAttribute.BASS_ATTRIB_VOL, 0, 500)
        While Bass.BASS_ChannelIsSliding(Stream, Un4seen.Bass.BASSAttribute.BASS_ATTRIB_VOL)
          Application.DoEvents()
        End While
      End If
    End If
    If Stream <> 0 Then
      Bass.BASS_ChannelStop(Stream)
    End If
    tmrAudio.Enabled = False
    Bass.BASS_StreamFree(Stream)
    TextScroller.Texts = " "
    TextScroller.StopScroller()
    PositionTimer.CurrentTimeInSeconds = 0

    m_UpdatingUi = True
    tbProgress.Value = 0
    m_UpdatingUi = False

    m_bPlaying = False
    m_bPaused = False
  End Sub

  Private Sub PauseAudio()
    If m_bPlaying = False Then Exit Sub

    If m_bPaused = False Then
      m_bPaused = True
      Bass.BASS_ChannelPause(Stream)
    Else
      m_bPaused = False
      Bass.BASS_ChannelPlay(Stream, False)
    End If
  End Sub

#End Region

  Private Sub frmAudioPlayer_DragOver(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragOver
    HandleDragAndDrop(e)
  End Sub

  Private Sub frmAudioPlayer_DragEnter(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragEnter
    HandleDragAndDrop(e)
  End Sub

  Private Sub HandleDragAndDrop(ByVal e As DragEventArgs)
    If e.Data.GetDataPresent("AudioTrack[]") Then
      e.Effect = DragDropEffects.Move
    ElseIf e.Data.GetDataPresent(DataFormats.FileDrop) Then
      If (e.AllowedEffect And DragDropEffects.Move) = DragDropEffects.Move Then
        e.Effect = DragDropEffects.Move
      Else
        e.Effect = DragDropEffects.Copy
      End If
    End If
  End Sub

  Private Sub frmAudioPlayer_DragDrop(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragDrop
    If e.Data.GetDataPresent("AudioTrack[]") Then
      'Drag'n drop from Tracklist
      Dim tracks() As AudioTrack = CType(e.Data.GetData("AudioTrack[]"), AudioTrack())
      m_Playlist.Clear()
      For i As Integer = tracks.Length - 1 To 0 Step -1
        m_Playlist.Push(tracks(i).FileName)
      Next
      PlayAudio(m_Playlist.Pop())
    ElseIf e.Data.GetDataPresent(DataFormats.FileDrop) Then
      'Drag'n drop from any file
      Dim k() As String = CType(e.Data.GetData(DataFormats.FileDrop), String())
      If k Is Nothing Then Return ' invalid file drop

      m_Playlist.Clear()
      For i As Integer = k.Length - 1 To 0 Step -1
        m_Playlist.Push(k(i))
      Next

      PlayAudio(m_Playlist.Pop())
    End If
  End Sub

  Private Sub tmrAudio_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrAudio.Tick
    Dim isActive As Integer = Bass.BASS_ChannelIsActive(Stream)
    'Stream Played to the end?
    If (isActive = Un4seen.Bass.BASSActive.BASS_ACTIVE_STOPPED) Then
      StopAudio()
      If m_Playlist.Count > 0 Then
        PlayAudio(m_Playlist.Pop())
      End If
      Exit Sub
    End If
    'We're paused?
    If (isActive = Un4seen.Bass.BASSActive.BASS_ACTIVE_PAUSED) Then
      Exit Sub
    End If

    'Not stopped or paused update UI
    Dim streamLength As Long = Bass.BASS_ChannelGetLength(Stream)
    Dim streamPos As Long = Bass.BASS_ChannelGetPosition(Stream)
    Dim tElapsed As Double = Bass.BASS_ChannelBytes2Seconds(Stream, streamPos)
    Dim level As Integer = Bass.BASS_ChannelGetLevel(Stream)
    PositionTimer.CurrentTimeInSeconds = Convert.ToInt32(tElapsed)

    m_UpdatingUi = True
    tbProgress.Value = Convert.ToInt32(Math.Max(0, Math.Min(50, streamPos / streamLength * 100 / 2)))
    m_UpdatingUi = False
  End Sub

  Private Sub cmdPlay_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdPlay.Click
    If m_bPaused Then
      Me.cmdPlay.Image = Global.CDBurnerXP_Pro.My.Resources.Resources.AudioPlayerPause
      PauseAudio()
    ElseIf m_bPlaying Then
      Me.cmdPlay.Image = Global.CDBurnerXP_Pro.My.Resources.Resources.AudioPlayerPlay
      PauseAudio()
    Else
      PlayAudio(String.Empty)
    End If
  End Sub

  Private Sub tbProgress_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tbProgress.ValueChanged
    If Not m_bPlaying Or m_UpdatingUi Then Exit Sub

    Dim vol As Single
    Bass.BASS_ChannelGetAttribute(Stream, Un4seen.Bass.BASSAttribute.BASS_ATTRIB_VOL, vol)
    Bass.BASS_ChannelSlideAttribute(Stream, Un4seen.Bass.BASSAttribute.BASS_ATTRIB_VOL, 0, 50)
    While Bass.BASS_ChannelIsSliding(Stream, Un4seen.Bass.BASSAttribute.BASS_ATTRIB_VOL)
    End While

    Bass.BASS_ChannelSetPosition(Stream, Convert.ToInt64((tbProgress.Value / 50) * Bass.BASS_ChannelGetLength(Stream)))
    Bass.BASS_ChannelSlideAttribute(Stream, Un4seen.Bass.BASSAttribute.BASS_ATTRIB_VOL, 1, 50)

    While Bass.BASS_ChannelIsSliding(Stream, Un4seen.Bass.BASSAttribute.BASS_ATTRIB_VOL)
    End While
    PositionTimer.CurrentTimeInSeconds = Convert.ToInt32(Bass.BASS_ChannelBytes2Seconds(Stream, Bass.BASS_ChannelGetPosition(Stream)))
  End Sub

  Private Sub frmAudioPlayer_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
    If e.CloseReason = CloseReason.UserClosing Then
      e.Cancel = True
      Me.Hide()
      If Not m_MyMenuItem Is Nothing Then
        m_MyMenuItem.Checked = False
      End If
    End If
  End Sub

  Private Sub frmAudioPlayer_ResizeEnd(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.ResizeEnd
    If Me.Width < 93 Then
      Me.Width = 93
    ElseIf Me.Width > 93 Then
      Me.Width = 357
    End If
  End Sub
End Class
