Imports System.IO
Imports System.Runtime.InteropServices
Imports Microsoft.Win32
Imports CDBurnerXP

Public Class ISOTools
  Public Enum ISOFormats
    ISO9660 = 0
    BIN = 1
  End Enum

  <DllImport("kernel32.dll")> Private Shared Function GetDriveType(ByVal nDrive As String) As Integer
  End Function
  <DllImport("kernel32.dll")> Private Shared Function ReadFile(ByVal hFile As Integer, ByVal Buffer As Byte(), ByVal nNumberOfBytesToRead As Integer, ByRef lpNumberOfBytesRead As Integer, ByVal lpOverlapped As Integer) As Integer
  End Function
  <DllImport("kernel32.dll")> Private Shared Function CreateFile(<MarshalAs(UnmanagedType.LPStr)> ByVal lpFileName As String, ByVal dwDesiredAccess As Integer, ByVal dwShareMode As Integer, ByVal lpSecurityAttributes As Integer, ByVal dwCreationDisposition As Integer, ByVal dwFlagsAndAttributes As Integer, ByVal hTemplateFile As Integer) As Integer
  End Function
  <DllImport("kernel32.dll")> Private Shared Function CloseHandle(ByVal hObject As Integer) As Integer
  End Function
  <DllImport("kernel32.dll")> Private Shared Function SetFilePointer(ByVal hFile As Integer, ByVal lDistanceToMove As Integer, ByRef lpDistanceToMoveHigh As Integer, ByVal dwMoveMethod As Integer) As Integer
  End Function
  Private Const DRIVE_UNKNOWN As Integer = 0
  Private Const DRIVE_CDROM As Integer = 5
  Private Const DRIVE_FIXED As Integer = 3
  Private Const DRIVE_RAMDISK As Integer = 6
  Private Const DRIVE_REMOVABLE As Integer = 2
  Private Const DRIVE_REMOTE As Integer = 4
  Private Const FILE_SHARE_READ As Integer = &H1
  Private Const FILE_SHARE_WRITE As Integer = &H2
  Private Const CREATE_NEW As Integer = 1
  Private Const OPEN_EXISTING As Integer = 3
  Private Const GENERIC_READ As Long = &H80000000
  Private Const GENERIC_WRITE As Long = &H40000000
  Private Const FILE_BEGIN As Integer = 0
  Private Const FILE_CURRENT As Long = 1
  Private _InputFilename As String
  Private _OutputFilename As String
  Private _ISOOutputFormat As ISOFormats
  Private _CancelExtract As Boolean = False
  Private _CancelConvert As Boolean = False
  Private m_ErrorMessage As String = String.Empty
  Public Event ConvertProgress(ByVal Percent As Double)
  Public Event ConvertCancelled()
  Public Event ExtractImageProgress(ByVal Percent As Single)
  Public Event ExtractCancelled()
  Public Event ExtractDone()
  Public Property CancelConvert() As Boolean
    Get
      Return _CancelConvert
    End Get
    Set(ByVal Value As Boolean)
      _CancelConvert = Value
    End Set
  End Property
  Public Property CancelExtract() As Boolean
    Get
      Return _CancelExtract
    End Get
    Set(ByVal Value As Boolean)
      _CancelExtract = Value
    End Set
  End Property
  Public Property OutputFilename() As String
    Get
      Return _OutputFilename
    End Get
    Set(ByVal Value As String)
      _OutputFilename = Value
    End Set
  End Property
  Public Property InputFilename() As String
    Get
      Return _InputFilename
    End Get
    Set(ByVal Value As String)
      _InputFilename = Value
    End Set
  End Property

  Public ReadOnly Property ErrorMessage() As String
    Get
      Return m_ErrorMessage
    End Get
  End Property

  Public Property ISOOutputFormat() As ISOFormats
    Get
      Return _ISOOutputFormat
    End Get
    Set(ByVal Value As ISOFormats)
      _ISOOutputFormat = Value
    End Set
  End Property
  Public Function DoBin2DVDISO() As Boolean
    Dim inputfs As FileStream
    Dim outputfs As FileStream
    Dim buf(182) As Byte
    Dim header As Integer = 12
    Dim edc As Integer = 4
    Dim pi As Integer = 10
    Dim dvd_PIsize As Integer = 10
    Dim blocksize As Integer = 37856
    'First algorith try
    '1.Read 34944 bytes from source
    '2.skip the first 12 of these
    '3.write the next 160 bytes
    '4.skip 10bytes
    '5.for 10 times write 172bytes skip 10bytes
    '6.write 168bytes skip 14bytes
    '7.repeat 16 times
    '8.move file pointer 2912bytes
    '9.repeat until read bytes=total bytes
    'Each 182 bytes read consist of 172 framedata and 10 pi info
    'ie first 182 bytes is header+data+pi
    'next is data+pi
    '.
    '.
    '.
    'last(12) is data+edc+pi
    '16 of these superframes together with PO data makes an block(12*16*182=34944)
    'each block is 16 superframes+16POframes(172*16=)
    'Read a full eccblock of 37865 bytes
    'strip the PO+PI data ie first the last 2912 bytes ie read 37856-2912=34953 bytes then jump 2912 bytes
    'now we have the superframe data left first skip the header 12bytes then read 160 data and write
    'then read 172 data write data jump 10 10times
    'finally read 168 data write data jump 14
    'read next eccblock and repeat

    Dim i As Long = 0
    Try
      inputfs = New FileStream(_InputFilename, FileMode.Open, FileAccess.Read, FileShare.Read)
    Catch ex As Exception
      MessageBox.Show(ex.Message, "File Input IO Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
      Return False
    End Try
    Try
      outputfs = New FileStream(_OutputFilename, FileMode.OpenOrCreate, FileAccess.Write, FileShare.Write)
    Catch ex As Exception
      MessageBox.Show(ex.Message, "File Output IO Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
      Return False
    End Try
    Dim source_length As Long = 0
    source_length = Convert.ToInt64(inputfs.Length / blocksize)
    inputfs.Seek(0, SeekOrigin.Begin)
    While i < source_length And (Not _CancelConvert)
      For p As Integer = 0 To 15
        For k As Integer = 0 To 11
          inputfs.Read(buf, 0, 182)
          outputfs.Write(buf, 12, 160)
          For j As Integer = 0 To 9
            inputfs.Read(buf, 0, 182)
            outputfs.Write(buf, 0, 172)
          Next
          inputfs.Read(buf, 0, 182)
          outputfs.Write(buf, 0, 168)
        Next k
      Next p
      inputfs.Seek(2912, SeekOrigin.Current)
      i += 1
      RaiseEvent ConvertProgress((i * 100) / source_length)
      Application.DoEvents()
    End While

    outputfs.Flush()
    inputfs.Close()
    outputfs.Close()
    If _CancelConvert = True Then
      RaiseEvent ConvertCancelled()
      Return False
    End If
    Return True
  End Function
  Public Function ConvertImages() As Boolean
    Select Case Path.GetExtension(_InputFilename).ToLower
      Case ".nrg"
        Return Nrg2ISO()
      Case ".bin"
        Return Bin2ISO()
        'Case ".b5i"
        '  Return B5I2ISO()
      Case Else
        Return False
    End Select
  End Function
  Public Function B5I2ISO() As Boolean
    Dim inputfs As FileStream = Nothing
    Try
      inputfs = New FileStream(_InputFilename, FileMode.Open, FileAccess.Read, FileShare.Read)
    Catch ex As Exception
      MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
      inputfs.Close()
      Return False
    End Try
    inputfs.Close()
    Return DoB5I2CDISO()
  End Function
  Public Function Nrg2ISO() As Boolean
    Dim inputfs As FileStream = Nothing
    Try
      inputfs = New FileStream(_InputFilename, FileMode.Open, FileAccess.Read, FileShare.Read)
    Catch ex As Exception
      MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
      inputfs.Close()
      Return False
    End Try
    inputfs.Close()
    Return DoNrg2CDISO()
  End Function
  Public Function Bin2ISO() As Boolean
    Dim inputfs As FileStream = Nothing
    Dim DVDHEADER() As Byte = {&H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0}
    Try
      inputfs = New FileStream(_InputFilename, FileMode.Open, FileAccess.Read, FileShare.Read)
    Catch ex As Exception
      MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
      inputfs.Close()
      Return False
    End Try
    Dim buf(16) As Byte
    inputfs.Read(buf, 0, 16)
    inputfs.Close()
    If ArrayCompare(buf, DVDHEADER, 12) Then
      Return DoBin2DVDISO()
    Else
      Return DoBin2CDISO()
    End If
  End Function

  Private Function DoNrg2CDISO() As Boolean
    Dim inputfs As FileStream = Nothing
    Dim outputfs As FileStream = Nothing
    Const BLOCK_SIZE As Integer = 1048576
    Try
      inputfs = New FileStream(_InputFilename, FileMode.Open, FileAccess.Read, FileShare.Read)
    Catch ex As Exception
      MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
      inputfs.Close()
      Return False
    End Try
    Try
      outputfs = New FileStream(_OutputFilename, FileMode.OpenOrCreate, FileAccess.Write, FileShare.Write)
    Catch ex As Exception
      MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
      inputfs.Close()
      If Not outputfs Is Nothing Then
        outputfs.Close()
      End If
      Return False
    End Try
    Dim totalbytes As Long = 0
    Dim bytesleft As Long = inputfs.Length
    Dim bytestoread As Integer = 0
    Dim bytesread As Integer = 0
    Dim buffer(BLOCK_SIZE) As Byte
    Dim l_part(20) As Byte
    inputfs.Seek(-20, SeekOrigin.End)
    inputfs.Read(l_part, 0, 20)

    If (BitConverter.ToInt32(l_part, 0) = 558124613 And BitConverter.ToInt32(l_part, 8) = 894584142) Or _
       (BitConverter.ToInt32(l_part, 4) = 558124613 And BitConverter.ToInt32(l_part, 12) = 1330791758) Then
      bytesleft = System.Net.IPAddress.NetworkToHostOrder(BitConverter.ToInt32(l_part, 16))
    End If
    bytesleft -= 512 * 600
    totalbytes = bytesleft
    inputfs.Seek(512 * 600, SeekOrigin.Begin)
    While bytesleft > 0 And (Not _CancelConvert)
      If bytesleft > BLOCK_SIZE Then
        bytestoread = BLOCK_SIZE
      Else
        bytestoread = Convert.ToInt32(bytesleft)
      End If
      bytesread = inputfs.Read(buffer, 0, bytestoread)
      If bytesread = 0 Then Exit While
      bytesleft -= bytesread
      outputfs.Write(buffer, 0, bytesread)
      RaiseEvent ConvertProgress(((totalbytes - bytesleft) / totalbytes) * 100)
      Application.DoEvents()
    End While
    outputfs.Flush()
    outputfs.Close()
    inputfs.Close()
    If _CancelConvert = True Then
      RaiseEvent ConvertCancelled()
      Return False
    End If
    Return True
  End Function
  Private Function DoB5I2CDISO() As Boolean
    Dim inputfs As FileStream = Nothing
    Dim outputfs As FileStream = Nothing
    Dim HEADER_BYTES() As Byte = {&HFF, &HFF, &HFF, &HFF, &HFF, &HFF, &HFF, &HFF, &HFF, &HFF, &HFF, &HFF}
    Dim buf(2447) As Byte
    Dim header(15) As Byte
    'Dim seek_header As Integer
    Dim seek_ecc As Integer
    Dim sector_size As Integer
    Dim bw As Long = 1, i As Long = 0
    Dim source_length As Long = 0
    Try
      inputfs = New FileStream(_InputFilename, FileMode.Open, FileAccess.Read, FileShare.Read)
    Catch ex As Exception
      MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
      inputfs.Close()
      Return False
    End Try
    Try
      outputfs = New FileStream(_OutputFilename, FileMode.OpenOrCreate, FileAccess.Write, FileShare.Write)
    Catch ex As Exception
      MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
      outputfs.Close()
      Return False
    End Try
    inputfs.Seek(2352, SeekOrigin.Begin)
    inputfs.Read(buf, 0, 16)
    If Not ArrayCompare(buf, HEADER_BYTES, 12) Then
      seek_ecc = 384
      sector_size = 2448
    Else
      seek_ecc = 288
      sector_size = 2352
    End If
    source_length = Convert.ToInt64(inputfs.Length / sector_size)
    inputfs.Seek(0, SeekOrigin.Begin)
    While i < source_length And (Not _CancelConvert)
      inputfs.Seek(16, SeekOrigin.Current)
      inputfs.Read(buf, 0, 2048)
      If bw > 150 Then
        outputfs.Write(buf, 0, 2048)
      End If
      inputfs.Seek(seek_ecc, SeekOrigin.Current)
      bw += 1
      RaiseEvent ConvertProgress((i * 100) / source_length)
      Application.DoEvents()
      i += 1
    End While
    outputfs.Flush()
    outputfs.Close()
    inputfs.Close()
    If _CancelConvert = True Then
      RaiseEvent ConvertCancelled()
      Return False
    End If
    Return True
  End Function
  Private Function DoBin2CDISO() As Boolean
    Dim inputfs As FileStream = Nothing
    Dim outputfs As FileStream = Nothing
    Dim HEADER_BYTES() As Byte = {&H0, &HFF, &HFF, &HFF, &HFF, &HFF, &HFF, &HFF, &HFF, &HFF, &HFF, &H0}
    Dim buf(2352) As Byte
    Dim header(16) As Byte
    Dim seek_header As Integer
    Dim seek_ecc As Integer
    Dim sector_size As Integer
    Dim i As Long = 0
    Dim source_length As Long = 0
    Try
      inputfs = New FileStream(_InputFilename, FileMode.Open, FileAccess.Read, FileShare.Read)
    Catch ex As Exception
      MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
      inputfs.Close()
      Return False
    End Try
    Try
      outputfs = New FileStream(_OutputFilename, FileMode.OpenOrCreate, FileAccess.Write, FileShare.Write)
    Catch ex As Exception
      MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
      If Not outputfs Is Nothing Then
        outputfs.Close()
      End If
      Return False
    End Try
    inputfs.Read(buf, 0, 16)

    If Not ArrayCompare(buf, HEADER_BYTES, 12) Then
      seek_header = 8
      seek_ecc = 280
      sector_size = 2336
    Else
      Select Case buf(15)
        Case 2
          seek_header = 24   ' Mode2/2352    
          seek_ecc = 280
          sector_size = 2352
        Case 1
          seek_header = 16    'Mode1/2352
          seek_ecc = 288
          sector_size = 2352
        Case Else
          MessageBox.Show("Track format unsupported", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
          Return False
      End Select
    End If
    source_length = Convert.ToInt64(inputfs.Length / sector_size)
    inputfs.Seek(0, SeekOrigin.Begin)
    While i < source_length And (Not _CancelConvert)
      inputfs.Position = inputfs.Position + seek_header
      inputfs.Read(buf, 0, 2048)
      outputfs.Write(buf, 0, 2048)
      inputfs.Position = inputfs.Position + seek_ecc
      i += 1
      RaiseEvent ConvertProgress((i * 100) / source_length)
      Application.DoEvents()
    End While

    outputfs.Flush()
    inputfs.Close()
    outputfs.Close()
    If _CancelConvert = True Then
      RaiseEvent ConvertCancelled()
      Return False
    End If
    Return True
  End Function
  Private Function ArrayCompare(ByVal a1 As Byte(), ByVal a2 As Byte(), ByVal cmplength As Integer) As Boolean
    Dim i As Integer = 0
    While i < cmplength AndAlso a1(i) = a2(i)
      i += 1
    End While
    Return (i = cmplength)
  End Function
  Public Function isFloppy(ByVal Driveletter As String) As Boolean
    Select Case GetDriveType(Driveletter)
      Case DRIVE_REMOVABLE
        Return True
      Case Else
        Return False
    End Select
  End Function
  Public Sub ReadImageFromFloppyDevice(ByVal DriveLetter As String, ByVal ImagePath As String)

    Dim outputfs As New FileStream(ImagePath, FileMode.OpenOrCreate, FileAccess.Write, FileShare.ReadWrite)
    Dim bytesread As Integer = 1
    Dim i As Long = 0
    Dim devname As String = "\\.\" & DriveLetter.Substring(0, 1) & ":"
    Dim devhandle As Integer
    Try
      devhandle = CreateFile(devname, GENERIC_READ, FILE_SHARE_READ Or FILE_SHARE_WRITE, 0, OPEN_EXISTING, 0, 0)
    Catch
      outputfs.Close()
      Throw New Exception("Could not open Drive")
    End Try
    If devhandle = -1 Then
      CloseHandle(devhandle)
      outputfs.Close()
      Throw New Exception("Could not open Drive")
      Exit Sub
    End If
    Select Case GetDriveType(DriveLetter)
      Case DRIVE_CDROM
        'CDROM
      Case DRIVE_REMOVABLE
        'Floppy,Zip etc
      Case Else
        Exit Sub
    End Select

    Dim lpFreeBytesAvailable As Long
    Dim lpTotalNumberOfBytes As Long
    Dim lpTotalNumberOfFreeBytes As Long
    Kernel32.GetDiskFreeSpaceEx(DriveLetter, lpFreeBytesAvailable, lpTotalNumberOfBytes, lpTotalNumberOfFreeBytes)
    Dim buffer(1024 * 10) As Byte
    SetFilePointer(devhandle, 0, 0, FILE_BEGIN)

    bytesread = 1
    Dim percentage As Double
    Dim totalbyteswritten As Long = 0
    While bytesread > 0 And ReadFile(devhandle, buffer, 1024, bytesread, Nothing) > 0 And (Not _CancelExtract)
      outputfs.Write(buffer, 0, bytesread)
      totalbyteswritten += bytesread
      percentage = (totalbyteswritten / lpTotalNumberOfBytes) * 100
      RaiseEvent ExtractImageProgress(Convert.ToSingle(Math.Min(percentage, 100)))
      Application.DoEvents()
    End While

    CloseHandle(devhandle)
    outputfs.Close()
    If _CancelExtract = True Then
      RaiseEvent ExtractCancelled()
    Else
      RaiseEvent ExtractDone()
    End If


  End Sub

  Public Function ReadImageFromCDDevice(ByVal DriveLetter As String, ByVal ImagePath As String) As Boolean
    Dim outputfs As FileStream
    Try
      outputfs = New FileStream(ImagePath, FileMode.OpenOrCreate, FileAccess.Write, FileShare.ReadWrite)
    Catch ex As IOException
      m_ErrorMessage = "Failed saving ISO image, cannot write to the target location. The file might be in use by another application."
      Return False
    Catch ex As UnauthorizedAccessException
      m_ErrorMessage = "Extracting data from the source device failed, because you do not have sufficient permissions."
      Return False
    End Try

    Dim bytesread As Integer = 1
    Dim devname As String = "\\.\" & DriveLetter.Substring(0, 1) & ":"
    Dim devhandle As Integer
    Try
      devhandle = CreateFile(devname, GENERIC_READ, FILE_SHARE_READ Or FILE_SHARE_WRITE, 0, OPEN_EXISTING, 0, 0)
    Catch
      m_ErrorMessage = "Failed to read from the source device."
      outputfs.Close()
      Return False
    End Try
    If devhandle = -1 Then
      CloseHandle(devhandle)
      outputfs.Close()
      m_ErrorMessage = "Failed to read from the source device."
      Return False
    End If
    Select Case GetDriveType(DriveLetter)
      Case DRIVE_CDROM
        'CDROM
      Case Else
        m_ErrorMessage = "Failed to read from the source device."
        outputfs.Close()
        Return False
    End Select

    Dim lpFreeBytesAvailable As Long ' should always be zero here
    Dim lpTotalNumberOfBytes As Long
    Dim lpTotalNumberOfFreeBytes As Long
    Kernel32.GetDiskFreeSpaceEx(DriveLetter, lpFreeBytesAvailable, lpTotalNumberOfBytes, lpTotalNumberOfFreeBytes)
    Dim buffer(131072 - 1) As Byte '65536 '1048576

    Dim totalbyteswritten As Long = 0
    SetFilePointer(devhandle, 0, 0, FILE_BEGIN)

    bytesread = 1
    While bytesread > 0 And ReadFile(devhandle, buffer, 131072, bytesread, Nothing) > 0 And (Not _CancelExtract)
      outputfs.Write(buffer, 0, bytesread)
      totalbyteswritten += bytesread
      RaiseEvent ExtractImageProgress(Convert.ToSingle((totalbyteswritten / lpTotalNumberOfBytes) * 100))
      Application.DoEvents()
    End While
    outputfs.Flush()

    If _CancelExtract = True Then
      CloseHandle(devhandle)
      outputfs.Close()
      RaiseEvent ExtractCancelled()
      Return False
    End If

    CloseHandle(devhandle)
    outputfs.Close()
    Return True
  End Function
  Public Function GetFreeSpace(ByVal Drivename As String) As Long
    Dim lBytesTotal, lFreeBytes, lFreeBytesAvailable As Long

    Dim iAns As Long

    iAns = Kernel32.GetDiskFreeSpaceEx(Drivename, lFreeBytesAvailable, _
         lBytesTotal, lFreeBytes)
    If iAns > 0 Then
      Return lFreeBytes
    Else
      Throw New IOException("Invalid or unreadable drive")
    End If
  End Function
  Public Function GetTotalSpace(ByVal Drivename As String) As Long
    Dim lBytesTotal, lFreeBytes, lFreeBytesAvailable As Long

    Dim iAns As Long

    iAns = Kernel32.GetDiskFreeSpaceEx(Drivename, lFreeBytesAvailable, _
         lBytesTotal, lFreeBytes)
    If iAns > 0 Then

      Return lBytesTotal
    Else
      Throw New IOException("Invalid or unreadable drive")
    End If
  End Function
  Public Function isISOCD(ByVal Drivename As String) As Boolean
    Dim devname As String = "\\.\" & Drivename.Substring(0, 1) & ":"
    Dim devhandle As Integer
    Dim bytesread As Integer
    Try
      devhandle = CreateFile(devname, GENERIC_READ, FILE_SHARE_READ Or FILE_SHARE_WRITE, 0, OPEN_EXISTING, 0, 0)
    Catch
      Throw New Exception("Could not open Drive")
    End Try
    If devhandle = -1 Then
      CloseHandle(devhandle)
      Throw New Exception("Could not open Drive")
      Exit Function
    End If
    Dim buf(34816) As Byte
    ReadFile(devhandle, buf, 34816, bytesread, Nothing)
    CloseHandle(devhandle)

    If buf(32774) <> 1 OrElse buf(32775) <> 0 Then
      Return False
    End If
    ' the identifier would usually be CD001, but there seem to be others, like BEA01. Thus we simply check for chars.
    For i As Integer = 32769 To 32773
      If Not (buf(i) >= 48 AndAlso buf(i) <= 90) Then
        Return False
      End If
    Next
    Return True
  End Function


End Class
Module ISOFunctions
  Public Function IsNrgAudioImage(ByVal filename As String) As Boolean
    Using input As FileStream = File.OpenRead(filename)
      ' The last 12 bytes contain NERO and 4 Bytes for the footer start
      ' or the last 8 bytes contain NER5 and 8 bytes for the footer start
      Dim footer(12 - 1) As Byte
      input.Seek(-12, SeekOrigin.End)
      input.Read(footer, 0, footer.Length)
      Dim footerStart As Long

      Dim chunkId As String = Utility.ConvertBytesToString(footer, 0, 4)

      If chunkId = "NER5" Then
        footerStart = Utility.ConvertBytesToInt64(footer, 4)
      Else
        footerStart = Utility.ConvertBytesToInt64(footer, 8)
      End If

      ' Now check whether the first 4 bytes of the complete footer are CUES or CUEX
      input.Seek(footerStart, SeekOrigin.Begin)
      Dim cueIdentifier(4 - 1) As Byte
      input.Read(cueIdentifier, 0, cueIdentifier.Length)

      Dim cueId As String = Utility.ConvertBytesToString(cueIdentifier)
      Return cueId = "CUES" OrElse cueId = "CUEX"
    End Using
  End Function


End Module
