'*************************************************************************
' Klass: frmCDDVDBurnIso
' Beroenden: 
' Syfte: 
' F�rfattare: Fredrik Haglund, Stefan Haglund
' Senast �ndrad av: Stefan Haglund
' Senast �ndrad: 2007-04-24 18:06
' Kommentarer: 
'*************************************************************************

Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports System.IO
Imports CDBurnerXP
Imports CDBurnerXP.Controls
Imports CDBurnerXP.Net
Imports CDBurnerXP.Burning
Imports CDBurnerXP.Forms
Imports Microsoft.Win32
Imports System.Runtime.InteropServices
Imports RocketDivision.StarBurnX
Imports CDBurnerXP.Windows.Forms

Public Class frmCDDVDBurnIso
  Private m_DataBurner As New CDBurnerXP.Burning.ImageBurner
  Private m_strISOFilename As String = ""
  Private strPercentWritten As String

  Protected Overrides ReadOnly Property UseRemainingCalculator() As Boolean
    Get
      Return False
    End Get
  End Property

  Protected Overrides ReadOnly Property StarBurnDevice() As RocketDivision.StarBurnX.Drive
    Get
      Return cdbBS.DeviceSelect.SelectedStarBurnDevice
    End Get
  End Property

  Public Event WriteComplete()

  <CLSCompliant(False)> Public Sub New(Optional ByVal deviceLetter As String = Nothing, Optional ByVal ISOFileName As String = "")
    ' This call is required by the Windows Form Designer.
    InitializeComponent()

    ' Add any initialization after the InitializeComponent() call.
    AddHandler cdbBS.DeviceSelect.SelectedIndexChanged, AddressOf cbDriveSelect_SelectedIndexChanged
    AddHandler m_DataBurner.OnProgress, AddressOf HandleProgress
    AddHandler m_DataBurner.OnVerify, AddressOf HandleVerify
    AddHandler m_DataBurner.OnCloseSessionBegin, AddressOf HandleCloseSession
    AddHandler m_DataBurner.OnTestWriteDisabled, AddressOf HandleTestWriteDisabled

    If Not deviceLetter Is Nothing Then
      cdbBS.DeviceSelect.SelectedDeviceLetter = deviceLetter
    Else
      cdbBS.DeviceSelect.SelectedDeviceLetter = ApplicationSettings.IsoLastDeviceUsed
    End If

    m_strISOFilename = ISOFileName
  End Sub

  Private Sub SetTitle(ByVal text As String)
    Me.Text = text

    If Me.Owner Is Nothing Then
      Me.Text &= " - " & Application.ProductName
    End If
  End Sub

  Private Sub frmCDDVDBurnIso_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
    SetTitle(RM.GetString("Write ISO Image"))

    lblBurnMethod.Text = RM.GetString("Burn &method:")
    ' Adjust for translation size
    cboBurnMethod.Left = lblBurnMethod.Right + 6
    sepSettings.Text = RM.GetString("Burn Options")
    lblSelectPath.Text = RM.GetString("Select ISO Image To Burn:")
    chkMode2XA.Text = RM.GetString("Mode2XA instead of Mode1")
    chkPadding.Text = RM.GetString("&Pad DVD size to 1GB")
    toolTip.ToolTipTitle = chkPadding.Text.Replace("&", "")
    toolTip.SetToolTip(chkPadding, RM.GetString("To ensure compatibility with all DVD players, fill the DVD with 1GB of data," & vbCrLf & "even if the total size of the video files is lower than 1GB."))
    chkVerify.Text = RM.GetString("Verify Data After Burn")
    chkEject.Text = RM.GetString("Eject Disc After Burn")
    chkFinalizeDisc.Text = RM.GetString("Finalize Disc")
    cdbBS.BurnSpeedLabel = RM.GetString("Burning Speed:")
    lblStatus.Text = RM.GetString("Status: Idle")
    lblTime.Text = String.Format(RM.GetString("Time Elapsed (hh:mm:ss): {0}"), "")
    lblRemaining.Text = String.Format(RM.GetString("Time Remaining (hh:mm:ss): {0}"), "-")
    cmdStartBurn.Text = RM.GetString("Burn Disc")
    cmdCancel.Text = RM.GetString("Cancel")
    cmdConvertToISO.Text = RM.GetString("Convert to &ISO...")
    strPercentWritten = RM.GetString("{0}% Writing to disc")
    tabBurnOptions.Text = RM.GetString("Burn ISO Options")
    tabBurnProgress.Text = RM.GetString("Burn ISO Progress")
    chkShutdown.Text = RM.GetString("Shutdown computer after completion")
    lblTargetDevice.Text = RM.GetString("&Target device:")
    cmdSelectPath.Text = RM.GetString("Bro&wse...")

    m_AutoImportEnabled = False
    Animation.ImageList = MainToolStrip.Animations
    Animation.ShowFrame(0)

    ActivateControls(True)
    UpdateControlsForDriveCompatibility()

    txtISOPath.Text = m_strISOFilename

    ' Settings
    chkFinalizeDisc.Checked = ApplicationSettings.IsoFinalize
    chkMode2XA.Checked = ApplicationSettings.IsoMode2XA
    chkVerify.Checked = ApplicationSettings.DataBurnVerify
    chkEject.Checked = ApplicationSettings.IsoEjectDisc
    ' Set last used burn speed, if present
    cdbBS.SetLastWriteSpeed = ApplicationSettings.LastDataBurnSpeed

  End Sub

  Protected Overrides Sub ActivateControls(ByVal enable As Boolean)
    MyBase.ActivateControls(enable)

    cmdStartBurn.Enabled = enable
    cmdSelectPath.Enabled = enable
    txtISOPath.Enabled = enable
    cmdConvertToISO.Enabled = enable
    cdbBS.Enabled = enable
    cboBurnMethod.Enabled = enable
    chkFinalizeDisc.Enabled = enable
    chkMode2XA.Enabled = enable
    chkPadding.Enabled = enable
  End Sub

  Private Sub UpdateControlsForDriveCompatibility()
    If StarBurnDevice Is Nothing Then
      cboBurnMethod.Enabled = False
      Exit Sub
    End If

    cboBurnMethod.Items.Clear()
    cboBurnMethod.Items.Add(New TaggedComboBoxItem(RM.GetString("Choose automatically"), STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_AUTO))

    If (StarBurnDevice.DriveInfo.SupportedModesWrite And STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_TAO) > 0 Then
      cboBurnMethod.Items.Add(New TaggedComboBoxItem(RM.GetString("Track at Once"), STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_TAO))
    End If
    If (StarBurnDevice.DriveInfo.SupportedModesWrite And STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_DAO_96) > 0 Then
      cboBurnMethod.Items.Add(New TaggedComboBoxItem(RM.GetString("Disc at Once P-W (DAO96)"), STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_DAO_96))
    End If
    If (StarBurnDevice.DriveInfo.SupportedModesWrite And STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_DAO_16) > 0 Then
      cboBurnMethod.Items.Add(New TaggedComboBoxItem(RM.GetString("Disc at Once P+Q (DAO16)"), STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_DAO_16))
    End If
    If (StarBurnDevice.DriveInfo.SupportedModesWrite And STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_SAO) > 0 Then
      cboBurnMethod.Items.Add(New TaggedComboBoxItem(RM.GetString("Session at Once"), STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_SAO))
    End If
    If cboBurnMethod.Items.Count > 0 Then cboBurnMethod.SelectedIndex = 0
  End Sub

#Region "Drag and Drop ISO"

  Protected Overrides Sub OnDragEnter(ByVal drgevent As System.Windows.Forms.DragEventArgs)
    MyBase.OnDragEnter(drgevent)

    CheckDragDrop(drgevent)
  End Sub

  Protected Overrides Sub OnDragOver(ByVal drgevent As System.Windows.Forms.DragEventArgs)
    MyBase.OnDragOver(drgevent)

    CheckDragDrop(drgevent)
  End Sub

  Sub CheckDragDrop(ByVal drgevent As DragEventArgs)
    If drgevent.Data.GetDataPresent(DataFormats.FileDrop) Then
      drgevent.Effect = DragDropEffects.Copy
    End If
  End Sub

  Protected Overrides Sub OnDragDrop(ByVal drgevent As System.Windows.Forms.DragEventArgs)
    MyBase.OnDragDrop(drgevent)

    Dim files() As String = CType(drgevent.Data.GetData(DataFormats.FileDrop), String())
    If (files IsNot Nothing AndAlso files.Length > 0) Then
      txtISOPath.Text = files(0)
    End If
  End Sub

#End Region

  Private Sub cmdSelectPath_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSelectPath.Click
    With OpenISODialog
      .Title = RM.GetString("Select ISO File To Write")
      .Filter = RM.GetString("ISO File") & " (*.iso; *.mds)|*.iso; *.mds|" & RM.GetString("All files") & " (*.*)|*.*"
      .InitialDirectory = ApplicationSettings.LastUsedIsoPath
      If .ShowDialog(Me) = DialogResult.Cancel Then Exit Sub

      Try
        If File.Exists(.FileName) Then
          txtISOPath.Text = .FileName
          ApplicationSettings.LastUsedIsoPath = Path.GetDirectoryName(.FileName)
          Dim tmpstr As String = String.Format(RM.GetString("ISO File {0} Loaded"), Path.GetFileName(.FileName))
          lvwStatus.AddItem(LogListView.LogType.Information, tmpstr)
        Else
          Dim tmpstr As String = String.Format(RM.GetString("File {0} is not a valid ISO image."), Path.GetFileName(.FileName))
          MessageBox.Show(Me, tmpstr, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1)
          cmdSelectPath.PerformClick()
          Exit Sub
        End If
      Catch
        MessageBox.Show(Me, RM.GetString("Could not read source file from disc."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning)
      End Try

    End With
  End Sub

  Protected Overrides Sub OnStartBurnClick(ByVal sender As Object, ByVal e As System.EventArgs)
    MyBase.OnStartBurnClick(sender, e)

    If m_strISOFilename = "" Then
      MessageBox.Show(Me, RM.GetString("No ISO image has been loaded."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      Exit Sub
    End If

    Dim requiredSpace As Long = 0
    ' Check the file size and if we can access file file
    Try
      Using fsa As New FileStream(txtISOPath.Text, FileMode.Open, FileAccess.Read)
        requiredSpace = fsa.Length
      End Using
    Catch ex As Exception
      MessageBox.Show(Me, ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      Exit Sub
    End Try

    If Not CheckDisc.Show(Me, StarBurnDevice, CheckDisc.DiscFeature.Writable, CheckDisc.DiscType.Any, requiredSpace, False, False) Then
      Exit Sub
    End If


    m_StartTime = Now
    tmrElapsed.Enabled = True

    'Set the properties of the write
    m_DataBurner.Drive = StarBurnDevice
    m_DataBurner.NextSessionAllowed = (chkFinalizeDisc.Checked = False)
    m_DataBurner.ImageFileName = txtISOPath.Text
    m_DataBurner.Padding = chkPadding.Checked
    m_DataBurner.WriteSpeed = cdbBS.WriteSpeedLongValue

    Dim selectedMode As TaggedComboBoxItem = CType(cboBurnMethod.SelectedItem, TaggedComboBoxItem)
    If selectedMode Is Nothing Then
      m_DataBurner.Mode = STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_AUTO
    Else
      m_DataBurner.Mode = CType(selectedMode.Tag, STARBURN_WRITE_MODE)
    End If

    PrepareUiForWork()
    m_Status = Status.Burning

    Dim burningThread As New Thread(New ThreadStart(AddressOf StartBurnThread))
    burningThread.Start()
  End Sub

  Private Sub StartBurnThread()
    Try
      HandleBurnProcessStarted()

      Dim isMds As Boolean = (Path.GetExtension(m_DataBurner.ImageFileName).ToLower() = ".mds")
      m_DataBurner.BurnEx(False, isMds, IsChecked(chkMode2XA))

      If IsChecked(chkVerify) Then
        m_Status = Status.Verifying
        m_DataBurner.Verify()
      End If
    Catch ex As COMException
      Me.Invoke(New ParameterizedThreadStart(AddressOf HandleError), ex)
    Finally
      BurningEngine.TryUnlock(m_DataBurner.Drive)
      Me.BeginInvoke(New MethodInvoker(AddressOf StopWrite))
    End Try
  End Sub

  Protected Overrides Sub OnCancelClick(ByVal sender As Object, ByVal e As System.EventArgs)
    MyBase.OnCancelClick(sender, e)

    If m_Status <> Status.Idle Then
      ' Show message corresponding to the current work
      Dim message As String = RM.GetString("Are you sure that you want to cancel the current burning process?")
      If m_Status = Status.Verifying Then
        message = RM.GetString("Are you sure that you want to cancel the verify process?")
      End If

      If MessageBox.Show(Me, message, Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes Then
        lblStatus.Text = RM.GetString("Aborting write, please wait...")
        lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("The writing was cancelled by the user, aborting please wait.."))

        Dim cancelThread As New Thread(New ThreadStart(AddressOf CancelBurningProcess))
        cancelThread.Start()
      End If
    Else
      Me.Close()
    End If
  End Sub

  Private Sub CancelBurningProcess()
    m_DataBurner.Cancel()
    m_BurningProcessCancelled = True
  End Sub

  Protected Overrides Sub HandleBurnProcessStarted()
    MyBase.HandleBurnProcessStarted()

    SetTitle(String.Format(strPercentWritten, 0))
  End Sub

  Private Sub HandleVerify(ByVal percent As Integer, ByVal timeRemaining As Integer)
    If Animation.FirstFrame <> 124 Then '+1
      Animation.FirstFrame = 124 '+1
      Animation.LastFrame = 143 '+1
      Animation.Delay = 90
      Animation.Pause = 180
      Animation.StartAnimation()
      lblStatus.Text = RM.GetString("Status: Verifying data")
      lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Verification started"))
    End If

    SetTimeRemaining(TimeSpan.FromSeconds(timeRemaining))
    prgProgress.ShowMarquee = False
    prgProgress.Value = percent
    SetTitle(String.Format(RM.GetString("{0}% Verifying The Media - [{1}]"), percent, Path.GetFileName(m_strISOFilename)))

  End Sub

  Private Sub HandleProgress(ByVal percent As Integer, ByVal timeRemaining As Integer)
    If m_BurningProcessCancelled Then
      lblStatus.Text = RM.GetString("Aborting write, please wait...", Culture)
    Else
      If Animation.FirstFrame <> 1 Then '+1
        Animation.FirstFrame = 1 '+1
        Animation.LastFrame = 18 '+1
        Animation.Delay = 80
        Animation.Pause = 50
        Animation.StartAnimation()

        lblStatus.Text = RM.GetString("Writing disc...", Culture)
        lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Writing process started at {0}", Culture), cdbBS.WriteSpeedDescription))
      End If

      SetTimeRemaining(TimeSpan.FromSeconds(timeRemaining))

      prgProgress.Value = percent
      SetTitle(String.Format(strPercentWritten, percent))
    End If
  End Sub

  Protected Overrides Sub HandleError(ByVal ex As Object)
    MyBase.HandleError(ex, ApplicationSettings.LoggingEnabled, chkShutdown.Checked, m_DataBurner.LastErrorCode)
  End Sub

  Private Sub StopWrite()
    tmrElapsed.Enabled = False
    prgProgress.Value = 0
    prgProgress.ShowMarquee = False
    Dim verifyCancelled As Boolean = (m_Status = Status.Verifying AndAlso m_BurningProcessCancelled)
    m_Status = Status.Idle

    SetTitle(RM.GetString("Write ISO Image"))

    If m_BurningProcessCancelled AndAlso Not verifyCancelled Then
      If Animation.FirstFrame <> 74 Then '+1
        Animation.FirstFrame = 74 '+1
        Animation.LastFrame = 82 '+1
        Animation.Delay = 350
        Animation.Pause = 500
        Animation.StartAnimation()
      End If

      lblStatus.Text = RM.GetString("Writing cancelled...")
      lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Writing was cancelled by user"))

      'Completed Message
      TaskDialog.ShowOptionalMessageBox(Me, "IsoCancelled", Application.ProductName, "", RM.GetString("Writing was cancelled by user"), eTaskDialogButtons.OK, eSysIcons.Warning, 0)


    ElseIf m_DataBurner.LastErrorCode = STARBURN_RESULT_CODES.STARBURN_OPERATION_SUCCEEDED Then
      lblStatus.Text = RM.GetString("Write completed...")

      If Animation.FirstFrame <> 50 Then '+1
        Animation.FirstFrame = 50 '+1
        Animation.LastFrame = 61 '+1
        Animation.Delay = 90
        Animation.Pause = 500
        Animation.StartAnimation()
      Else
        Exit Sub
      End If

      ' Only show a hint about cancelled verification
      If verifyCancelled Then
        lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Verifying was cancelled by user."))
      End If

      lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Writing process is successfully completed at speed {0}."), cdbBS.WriteSpeedDescription))

      If chkEject.Checked Then
        m_DataBurner.Drive.Eject()
      End If

      If chkShutdown.Checked Then
        ExitWindows.ForceShutdown()
      End If

      Dim deltaTime As TimeSpan = Utility.TimeSpanSeconds(m_ElapsedTime - m_StartTime)
      Dim msg As String = String.Format(RM.GetString("Writing process is successfully completed at speed {0}."), cdbBS.WriteSpeedDescription) & Environment.NewLine & String.Format(RM.GetString("Total time for operation (hh:mm:ss): {0}"), deltaTime.ToString())
      TaskDialog.ShowOptionalMessageBox(Me, "IsoCompleted", Application.ProductName, "", msg, eTaskDialogButtons.OK, eSysIcons.Information, 0)
    End If

    ResetUiAfterWork()
  End Sub

  Private Sub txtISOPath_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtISOPath.TextChanged
    m_strISOFilename = txtISOPath.Text
  End Sub

  Private Sub cmdConvertToISO_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdConvertToISO.Click
    Using frmConvert As New frmCDDVDConvertISO
      frmConvert.ShowDialog(Me)
    End Using
  End Sub

  Private Sub cbDriveSelect_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs)
    UpdateControlsForDriveCompatibility()
  End Sub

  Protected Overrides Sub OnClosed(ByVal e As System.EventArgs)
    Animation.StopAnimation()
    ' Save settings...
    ApplicationSettings.IsoFinalize = chkFinalizeDisc.Checked
    ApplicationSettings.IsoMode2XA = chkMode2XA.Checked
    ApplicationSettings.LastDataBurnSpeed = cdbBS.WriteSpeedLongValue
    ApplicationSettings.IsoLastDeviceUsed = cdbBS.DeviceSelect.SelectedDeviceLetter
    ApplicationSettings.DataBurnVerify = chkVerify.Checked
    ApplicationSettings.IsoEjectDisc = chkEject.Checked
    m_AutoImportEnabled = True

    RemoveHandler m_DataBurner.OnProgress, AddressOf HandleProgress
    RemoveHandler m_DataBurner.OnVerify, AddressOf HandleVerify
    RemoveHandler m_DataBurner.OnCloseSessionBegin, AddressOf HandleCloseSession
    RemoveHandler m_DataBurner.OnTestWriteDisabled, AddressOf HandleTestWriteDisabled

    MyBase.OnClosed(e)

    Dispose(True)
  End Sub

End Class
