'*************************************************************************
' Klass: frmAudioCDCompilation
' Beroenden: 
' Syfte: 
' F�rfattare: Fredrik Haglund, Stefan Haglund
' Senast �ndrad av: Stefan Haglund
' Senast �ndrad: 2007-04-25 20:36
' Kommentarer: 
'*************************************************************************

Imports System.IO
Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports System.Runtime.InteropServices
Imports Bass = Un4seen.Bass.bass
Imports BassTags = Un4seen.Bass.AddOn.Tags.BassTags
Imports Microsoft.Win32
Imports CDBurnerXP
Imports CDBurnerXP.IO
Imports CDBurnerXP.Windows.Forms
Imports CDBurnerXP.Burning
Imports CDBurnerXP.Forms
Imports CDBurnerXP.Controls
Imports CDBurnerXP.Net
Imports RocketDivision

Public Class frmAudioCompilation
  Private m_FileReport As New FileAddErrorsDialog
  Public Shared AudioInstance As frmAudioCompilation
  Private m_MsgBoxOwner As Form = Me
  Private mDevice As StarBurnX.Drive
  Private mAudioImage As New AudioBurner
  Private m_NoAudioDevice As Boolean = False
  Private m_initDone As Boolean = False
  Private m_AudioPlayer As New frmAudioPlayer()
  Private m_ForceMedia As Single = 0
  Private WithEvents m_MainToolbar As MainToolStrip
  Private m_ExitApplicationAfterClose As Boolean = True

  Private Delegate Sub ErrorRecalcCompilationSizeDelegate(ByVal drive As StarBurnX.Drive, ByVal ex As COMException)

  Public Overrides ReadOnly Property ExitApplicationAfterClose() As Boolean
    Get
      Return m_ExitApplicationAfterClose
    End Get
  End Property

  Public Property MsgBoxOwner() As Form
    Get
      Return m_MsgBoxOwner
    End Get
    Set(ByVal Value As Form)
      m_MsgBoxOwner = Value
    End Set
  End Property

  Public Sub New()

    ' This call is required by the Windows Form Designer.
    InitializeComponent()

    m_MainToolbar = New MainToolStrip()
    Sizebar3.ContextMenu = cmuSizebar

    Me.Controls.Add(m_MainToolbar)

    tbNumCopies.NumericUpDown.Minimum = 1
    tbNumCopies.NumericUpDown.Maximum = 99
    tbNumCopies.NumericUpDown.TextAlign = HorizontalAlignment.Right
    tbNumCopies.NumericUpDown.Value = Math.Max(1, Math.Min(99, Conversion.ToInt(Settings.GetValue("frmAudioCDBurn", "NumberOfCopies", 1))))

    Me.Text = RM.GetString("Audio Compilation")
    filSelect.ShowHiddenItems = PathEx.IsShowHiddenFilesEnabled
    dirSelect.ShowHiddenObjects = PathEx.IsShowHiddenFilesEnabled
    sbrCompilationInfo.ImageList = imlDiscIcon
    m_AudioLayoutManager.CdaRipper = AddressOf RipAudioTracks
    filSelect.Filter = FileView.FilterType.Audio
  End Sub

  Protected Overrides Sub OnClosed(ByVal e As System.EventArgs)
    MyBase.OnClosed(e)

    RemoveHandler cboDrives.DeviceSelect.SelectedIndexChanged, AddressOf cboDrives_SelectedIndexChanged

    If m_AudioPlayer.Visible Then
      ApplicationSettings.AudioPlayerPos = m_AudioPlayer.Location
      ApplicationSettings.AudioPlayerSize = m_AudioPlayer.Size
    End If

    m_AudioPlayer.Dispose()

    'Clean up bass.net
    Bass.BASS_Free()

    'Settings
    ApplicationSettings.AudioShowPlayer = mnuAudioPlayer.Checked

    ApplicationSettings.AudioLastDeviceUsed = cboDrives.DeviceSelect.SelectedDeviceLetter

    ApplicationSettings.AudioExplorer = mnuExplorer.Checked
    ApplicationSettings.AudioTileVertical = mnuTileVertical.Checked
    ApplicationSettings.AudioSplitterExplorer = splExplorer.SplitterDistance
    ApplicationSettings.AudioSplitterVertical = splVertical.SplitterDistance

    If Directory.Exists(PathEx.GetTempRipPath()) Then
      PathEx.TryDeleteFiles(Directory.GetFiles(PathEx.GetTempRipPath()))
    End If

    If Not filSelect Is Nothing Then
      filSelect.SaveColumnLayout()
      ApplicationSettings.AudioExplorerViewStyle = filSelect.ViewStyle
      ApplicationSettings.AudioExplorerPath = filSelect.CurrentFolder
    End If
    Try
      ApplicationSettings.Save()
    Catch ex As Exception
    End Try

    'Clean up cd notify handlers
    MediaNotifier.Enabled = False

    If ExitApplicationAfterClose Then
      frmSelectProject.Instance.Close()
    Else
      frmSelectProject.Instance.Show()
    End If
  End Sub

  Protected Overrides Sub OnClosing(ByVal e As System.ComponentModel.CancelEventArgs)
    MyBase.OnClosing(e)

    If m_AudioLayoutManager.IsUnsaved Then
      Dim result As DialogResult = TaskDialog.ShowOptionalMessageBox(Me, "SaveAudioCompilation", RM.GetString("Save Compilation"), "", RM.GetString("Compilation has not been saved. Do you want to save the compilation before quitting?"), eTaskDialogButtons.YesNoCancel, eSysIcons.Warning, System.Windows.Forms.DialogResult.No)
      Select Case result
        Case DialogResult.Yes
          mnuSaveAs.PerformClick()
        Case DialogResult.Cancel
          e.Cancel = True
          Exit Sub
      End Select
    End If
  End Sub

  Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
    If (DesignMode) Then Exit Sub

    'Language
    mnuFile.Text = RM.GetString("&File")
    mnuOpen.Text = RM.GetString("Open...")
    mnuSave.Text = RM.GetString("Save")
    mnuSaveAs.Text = RM.GetString("Save As...")
    mnuImportFrom.Text = RM.GetString("Import fro&m")
    mnuPlaylist.Text = RM.GetString("&Playlist...")
    mnuCueSheet.Text = RM.GetString("&Cue sheet...")
    mnuAudioDisc.Text = RM.GetString("&Audio CD...")
    mnuPrintCover.Text = RM.GetString("Print Cover...")
    mnuPrintTrackList.Text = RM.GetString("Pri&nt track list...")
    mnuPrintTrackListPreview.Text = RM.GetString("Print track list pre&view...")
    mnuOptions.Text = RM.GetString("Op&tions")
    mnuExit.Text = RM.GetString("E&xit")
    mnuEdit.Text = RM.GetString("Edit")
    mnuClearCompilation.Text = RM.GetString("&Clear compilation")

    mnuRecorder.Text = RM.GetString("Recorder")
    mnuEject.Text = RM.GetString("Eject Disc")
    mnuCloseTray.Text = RM.GetString("Close Tray")
    mnuDiscInfo.Text = RM.GetString("Disc &information")

    mnuEraseDisc.Text = RM.GetString("Erase Disc...")
    mnuBurnDisc.Text = RM.GetString("Burn Disc...")
    mnuView.Text = RM.GetString("&View")
    mnuDisc.Text = RM.GetString("D&isc")
    mnuRecorderInfo.Text = RM.GetString("Device &information")
    mnuAudioPlayer.Text = RM.GetString("&Audio player")
    mnuExplorer.Text = RM.GetString("E&xplorer panel")
    mnuTile.Text = RM.GetString("&Tile")
    mnuDropBox.Text = RM.GetString("Show &dropbox")
    mnuTileHorizontal.Text = RM.GetString("&Horizontal")
    mnuTileVertical.Text = RM.GetString("&Vertical")
    mnuHelpOnLine.Text = RM.GetString("&Online help")
    mnuCheckVersion.Text = RM.GetString("&Check for new version")
    mnuWeb.Text = RM.GetString("CDBurnerXP on the &web")
    mnuAbout.Text = RM.GetString("&About CDBurnerXP")
    mnuHelp.Text = RM.GetString("Help")
    mnuClose.Text = RM.GetString("Close")
    tbMidBurn.Text = RM.GetString("Burn")
    tbMidErase.Text = RM.GetString("Erase")
    tbMidClear.Text = RM.GetString("Clear")
    tbMidAdd.Text = RM.GetString("Add")
    tbMidRemove.Text = RM.GetString("Re&move")
    Sizebar3.WarningText = RM.GetString("Data exceeds the size of the media")
    Sizebar3.UnitText = RM.GetString("MB")
    cmnuTracksClearCompilation.Text = RM.GetString("&Clear compilation")
    cmnuCDText.Text = RM.GetString("&CD-Text")
    mnuCDText.Text = RM.GetString("&CD-Text")
    cmnuTracksMoveUp.Text = RM.GetString("Move Up")
    cmnuTracksMoveDown.Text = RM.GetString("Move Down")
    cmnuTracksRemoveSelected.Text = RM.GetString("&Remove")
    cmnuTracksPaste.Text = RM.GetString("&Paste")
    cmnuTracksSelectAll.Text = RM.GetString("Select All")
    cmnuTracksPlayTrack.Text = RM.GetString("Play Track")
    mnuAddFiles.Text = RM.GetString("Add files...")
    mnuAddFolder.Text = RM.GetString("Add f&older...")
    mnuInvertSelection.Text = RM.GetString("Invert &selection")
    mnuFileFilter.Text = RM.GetString("&Filter files...")
    mnuSelectAll.Text = RM.GetString("Select &all")
    mnuReportDrive.Text = RM.GetString("Report Working Drive...")
    mnuRandomizeTracks.Text = RM.GetString("&Randomize tracks")
    cmnuRandomizeTracks.Text = RM.GetString("&Randomize tracks")
    tbLblCopies.Text = RM.GetString("&Number of copies:")
    lblDrivePic.ToolTipText = RM.GetString("Target device for the burning process")
    cboDrives.ToolTipText = RM.GetString("Target device for the burning process")

    cmnuSizebarForceDiscType.Text = RM.GetString("Select Media Size")
    cmnuSizebarRefreshDisc.Text = RM.GetString("Refresh Disc")
    cmnuSizebarAutoDetect.Text = RM.GetString("&Automatically detect media size")
    cmnuSizebarCD90.Text = String.Format(RM.GetString("CD {0}min ({1})"), "90", FormatFileSize.Format(800000000))
    cmnuSizebarCD80.Text = String.Format(RM.GetString("CD {0}min ({1})"), "80", FormatFileSize.Format(700000000))
    cmnuSizebarCD74.Text = String.Format(RM.GetString("CD {0}min ({1})"), "74", FormatFileSize.Format(650000000))
    cmnuSizebarCD63.Text = String.Format(RM.GetString("CD {0}min ({1})"), "63", FormatFileSize.Format(553000000))
    cmnuSizebarCD21.Text = String.Format(RM.GetString("CD {0}min ({1})"), "21", FormatFileSize.Format(185000000))

    AudioInstance = Me

    If Not BurningEngine.InitialiseBass() Then
      m_NoAudioDevice = True
      cmnuTracksPlayTrack.Visible = False
      cmnuTracksSep3.Visible = False
      mnuView.MenuItems.Remove(mnuAudioPlayer)
      m_AudioPlayer.Hide()
    End If

    AddHandler cboDrives.DeviceSelect.SelectedIndexChanged, AddressOf cboDrives_SelectedIndexChanged
    cboDrives_SelectedIndexChanged(Nothing, Nothing)
    cboDrives.DeviceSelect.SelectedDeviceLetter = ApplicationSettings.AudioLastDeviceUsed
    BeginRecalcCompilationSize(True)

    m_initDone = True
    'Settings
    dirSelect.FastModeSynchronization = True
    filSelect.CurrentFolder = ApplicationSettings.AudioExplorerPath
    dirSelect.FastModeSynchronization = False
    filSelect.ViewStyle = CType(ApplicationSettings.AudioExplorerViewStyle, LogicNP.FileViewControl.ViewStyles)
    mnuAudioPlayer.Checked = ApplicationSettings.AudioShowPlayer

    If ApplicationSettings.AudioTileVertical = True Then
      mnuTileVertical.PerformClick()
    End If

    If ApplicationSettings.AudioExplorer = False Then
      mnuExplorer.PerformClick()
    End If

    If Directory.Exists(PathEx.GetTempRipPath()) Then
      PathEx.TryDeleteFiles(Directory.GetFiles(PathEx.GetTempRipPath()))
    End If

    If Not AutoUpdate.IsOnlineUpdateAllowed Then
      mnuCheckVersion.Visible = False
      mnuSep5.Visible = False
    End If

    MyBase.OnLoad(e)

    If ApplicationSettings.AudioShowPlayer And Not m_NoAudioDevice Then
      m_AudioPlayer.MyMenuItem = mnuAudioPlayer
      m_AudioPlayer.Show(Me)
      If ApplicationSettings.AudioPlayerSize <> Size.Empty Then
        m_AudioPlayer.Size = ApplicationSettings.AudioPlayerSize
      End If

      If ApplicationSettings.AudioPlayerPos = Point.Empty Then
        ' sensible defaults
        m_AudioPlayer.Left = Me.Bounds.Right - m_AudioPlayer.Width - 20
        m_AudioPlayer.Top = Me.Bounds.Top + 20
      Else
        m_AudioPlayer.Location = ApplicationSettings.AudioPlayerPos
      End If
    End If

    AdjustSplitters()
  End Sub

  Private Function RipAudioTracks(ByVal paths() As String) As String()
    Using dialog As New RipTracksDialog(paths)
      dialog.ShowDialog(MsgBoxOwner)
      Return dialog.RippedPaths
    End Using
  End Function

  Protected Overrides Sub OnLoaded(ByVal sender As Object, ByVal e As System.EventArgs)
    LoadCompilation(g_sActionFile)
    g_sAction = ""
    g_sActionFile = ""
  End Sub

  Protected Overrides Sub OnMediaChanged(ByVal VolumeLabel As String)
    BeginRecalcCompilationSize(True)
  End Sub

  Private Sub MainToolbarClicked(ByVal sender As Object, ByVal e As System.Windows.Forms.ToolStripItemClickedEventArgs) Handles m_MainToolbar.ItemClicked
    If e.ClickedItem.Tag Is Nothing Then
      Exit Sub
    End If
    Select Case e.ClickedItem.Tag.ToString
      Case "Open"
        mnuOpen.PerformClick()
      Case "Save"
        mnuSave.PerformClick()
      Case "SaveAs"
        mnuSaveAs.PerformClick()
      Case "Close"
        m_ExitApplicationAfterClose = False
        Me.Close()
      Case "Print"
        mnuPrintCover.PerformClick()
      Case "Erase"
        mnuEraseDisc.PerformClick()
      Case "Burn"
        mnuBurnDisc.PerformClick()
      Case "Dropbox"
        mnuDropBox.PerformClick()
      Case "DiscInfo"
        mnuDiscInfo.PerformClick()
      Case "RecorderInfo"
        mnuRecorderInfo.PerformClick()
      Case "Eject"
        mnuEject.PerformClick()
      Case "CloseTray"
        mnuCloseTray.PerformClick()
    End Select
  End Sub

  ''' Refreshes the toolbar and sizebar
  Private Sub BeginRecalcCompilationSize(ByVal refreshDisc As Boolean)
    If mDevice Is Nothing Then Exit Sub

    If (refreshDisc) Then
      sbRemaining.Visible = False
      tsUpdatingDisc.Visible = True

      sbDisc.ImageIndex = 0
      sbDisc.Text = String.Empty
      sbDisc.BorderSides = ToolStripStatusLabelBorderSides.None
      sbTracksTime.BorderSides = ToolStripStatusLabelBorderSides.Left
    End If

    ' This information can be updated immediately
    Dim totalTime As TimeSpan = m_AudioLayoutManager.GetTotalLength(True)
    Dim totalTimeClean As TimeSpan = Utility.TimeSpanSeconds(totalTime)
    sbTracksTime.Text = FormatTranslation("Audio CD: {0} tracks, {1} play time", m_AudioLayoutManager.Tracks.Length, totalTimeClean.ToString())

    ThreadPool.QueueUserWorkItem(AddressOf RecalcCompilationSize, New Object() {mDevice, refreshDisc})
  End Sub

  Private Sub RecalcCompilationSize(ByVal params As Object)
    Dim drive As StarBurnX.Drive = CType(CType(params, Object())(0), StarBurnX.Drive)
    Dim refreshDisc As Boolean = CType(CType(params, Object())(1), Boolean)

    If drive Is Nothing Then Return

    Try
      If CBool(refreshDisc) Then drive.DiscInfo.Refresh()

      Me.BeginInvoke(New ParameterizedThreadStart(AddressOf AfterRecalcCompilationSize), drive)
    Catch ex1 As InvalidOperationException
      ' Form closed, no refresh necessary
    Catch ex As COMException
      Me.BeginInvoke(New ErrorRecalcCompilationSizeDelegate(AddressOf ErrorRecalcCompilationSize), drive, ex)
    Finally
      Try
        Me.BeginInvoke(New MethodInvoker(AddressOf FinishRecalcCompilationSize))
      Catch ex As InvalidOperationException
        ' Form closed, no refresh necessary
      End Try
    End Try
  End Sub

  Sub ErrorRecalcCompilationSize(ByVal drive As StarBurnX.Drive, ByVal ex As COMException)
    sbDisc.Text = RM.GetString("No Disc")
    sbDisc.ImageIndex = 1
    Sizebar3.OverDiscValue = 80 * 60
    Sizebar3.AdditionalWarningText = String.Format(RM.GetString("No usable disc present, assuming disc with {0} minutes capacity"), 80)

    If Not BaseForm.HandleStarBurnError(Me, ex, drive) Then
      Throw ex
    End If
  End Sub

  Sub AfterRecalcCompilationSize(ByVal pDevice As Object)
    Dim device As StarBurnX.Drive = CType(pDevice, StarBurnX.Drive)

    Try

      Dim onDiscText As String = String.Empty
      onDiscText = BurningEngine.GetDiscDescription(CType(device.DiscInfo.DiscType, StarBurnX.STARBURN_DISC_TYPE))
      Dim onDiscValue As Single = CSng(device.DiscInfo.UsedSize) / CheckDisc.AUDIO_SEC_PER_TRACK

      Dim noDiscWarning As String = String.Format(RM.GetString("No usable disc present, assuming disc with {0} minutes capacity"), 80)

      If device.DiscInfo.DiscType <> StarBurnX.STARBURN_DISC_TYPE.STARBURN_DISC_TYPE_NONE AndAlso device.LastErrorCode <> StarBurnX.STARBURN_RESULT_CODES.STARBURN_ERR_DEVICE_DISCONNECTED Then
        ' If disc is finalised, the sizebar will no longer be useful if we set the disc's information
        If device.DiscInfo.Status = StarBurnX.STARBURN_DISC_STATUS.STARBURN_DISC_STATUS_COMPLETE Then
          Sizebar3.OverDiscValue = 80 * 60
          onDiscValue = 0
          Sizebar3.AdditionalWarningText = noDiscWarning
        Else
          Sizebar3.AdditionalWarningText = String.Empty
          Sizebar3.OverDiscValue = CSng(device.DiscInfo.TotalSize) / CheckDisc.AUDIO_SEC_PER_TRACK
        End If

        ' Time used on disc
        Dim onDiscTime As TimeSpan = BurningEngine.GetAudioDiscDuration(device.DiscInfo.UsedSize)
        Dim onDiscTimeClean As TimeSpan = Utility.TimeSpanSeconds(onDiscTime)
        Dim onDiscBytes As Long = FormatFileSize.BlocksToAudioBytes(device.DiscInfo.UsedSize)
        If onDiscBytes = 0 Then
          onDiscText += " (" & RM.GetString("Empty") & ")"
        Else
          onDiscText += String.Format(" " & RM.GetString("(Used: {0} / {1})"), onDiscTimeClean.ToString(), FormatFileSize.Format(onDiscBytes))
        End If

        sbDisc.Text = onDiscText
      Else
        sbDisc.Text = RM.GetString("No Disc")
        sbDisc.ImageIndex = 1
        Sizebar3.OverDiscValue = 80 * 60
        Sizebar3.AdditionalWarningText = noDiscWarning
      End If

      If m_ForceMedia > 0 Then
        Sizebar3.AdditionalWarningText = String.Empty
        Sizebar3.OverDiscValue = m_ForceMedia
      End If

      Sizebar3.MaxValue = Sizebar3.OverDiscValue + 100

      ' Total time required
      Dim totalTime As TimeSpan = m_AudioLayoutManager.GetTotalLength(True)

      Sizebar3.NumberOfTracks = m_AudioLayoutManager.Tracks.Length
      Sizebar3.DurationArray = m_AudioLayoutManager.TrackTimes
      Sizebar3.BarValue = CSng(totalTime.TotalSeconds)
      Sizebar3.OnDiscValue = onDiscValue
      Sizebar3.UpdateSizeBar()

      ' Time free
      Dim freeTime As TimeSpan = BurningEngine.GetAudioDiscDuration(device.DiscInfo.TotalSize - device.DiscInfo.UsedSize)

      If device.DiscInfo.DiscType = StarBurnX.STARBURN_DISC_TYPE.STARBURN_DISC_TYPE_NONE Then
        sbRemaining.Text = RM.GetString("Remaining Size: N/A")
      Else
        Dim remainingTimeClean As TimeSpan = TimeSpan.FromSeconds(CInt((freeTime - totalTime).TotalSeconds()))
        Dim remainingBytes As Long = FormatFileSize.SecondsToBytes(CInt(remainingTimeClean.TotalSeconds()))
        sbRemaining.Text = String.Format(RM.GetString("Remaining Size: {0} ({1}MB)"), remainingTimeClean.ToString(), (remainingBytes / 1024 / 1024).ToString("0.00"))
      End If

      If String.IsNullOrEmpty(sbRemaining.Text) Then
        Sizebar3.AccessibleDescription = onDiscText.Trim()
      Else
        Sizebar3.AccessibleDescription = onDiscText.Trim() & ", " & sbRemaining.Text
      End If
    Catch ex As COMException
      ErrorRecalcCompilationSize(device, ex)
    End Try
  End Sub

  Public Sub FinishRecalcCompilationSize()
    If tsUpdatingDisc.Visible Then
      tsUpdatingDisc.Visible = False
      sbDisc.Visible = True
      sbDisc.BorderSides = ToolStripStatusLabelBorderSides.Right
      sbRemaining.Visible = True
      sbTracksTime.BorderSides = ToolStripStatusLabelBorderSides.Right
    End If
  End Sub

  Private Sub PreventSystemChanges(ByVal sender As Object, ByVal e As LogicNP.FolderViewControl.FolderViewDragDropEventArgs) Handles dirSelect.FolderViewDragDrop
    e.DoDefaultShellDropAction = Not ApplicationSettings.PreventDefaultDragDrop
  End Sub

  Private Sub PreventSystemChanges(ByVal sender As Object, ByVal e As LogicNP.FolderViewControl.FolderViewDragOverEventArgs) Handles dirSelect.FolderViewDragOver
    If ApplicationSettings.PreventDefaultDragDrop Then
      e.Effects = DragDropEffects.None
    End If
  End Sub

  Private Sub cboDrives_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs)
    If cboDrives.DeviceSelect.SelectedStarBurnDevice Is Nothing Then
      Exit Sub
    End If

    mDevice = cboDrives.DeviceSelect.SelectedStarBurnDevice

    'Set CDNotifyDevice and start notify
    If Not mDevice Is Nothing Then
      MediaNotifier.DriveLetter = mDevice.DriveInfo.Letter
      MediaNotifier.Enabled = True
      BeginRecalcCompilationSize(True)
    End If
  End Sub

  Private Sub mnuRandomize_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuRandomizeTracks.Click
    m_AudioLayoutManager.RandomizeTracks()
  End Sub

  Private Sub mnuExplorer_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuExplorer.Click
    If mnuExplorer.Checked Then
      mnuExplorer.Checked = False
      splVertical.Panel1Collapsed = True
      mnuTile.Enabled = False
    Else
      mnuExplorer.Checked = True
      splVertical.Panel1Collapsed = False
      mnuTile.Enabled = True
    End If
  End Sub

  Private Sub mnuAudioPlayer_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuAudioPlayer.Click
    If mnuAudioPlayer.Checked Then
      mnuAudioPlayer.Checked = False
      m_AudioPlayer.Hide()
    ElseIf Not m_NoAudioDevice Then
      ShowAudioPlayer()
    End If
  End Sub

  Private Sub mnuTileHorizontal_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuTileHorizontal.Click
    If splVertical.Orientation = Orientation.Vertical Then
      splVertical.Orientation = Orientation.Horizontal
      AdjustSplitters()
    End If
  End Sub

  Private Sub mnuTileVertical_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuTileVertical.Click
    If splVertical.Orientation = Orientation.Horizontal Then
      splVertical.Orientation = Orientation.Vertical
      AdjustSplitters()
    End If
  End Sub

  Private Sub ShowAudioPlayer()
    mnuAudioPlayer.Checked = True
    m_AudioPlayer.MyMenuItem = mnuAudioPlayer
    If Not m_AudioPlayer.Visible Then
      m_AudioPlayer.Show(Me)
    End If
  End Sub

  Private Sub BurnAudioCD()
    ' Check that device is selected
    If cboDrives.DeviceSelect.SelectedStarBurnDevice Is Nothing Then
      CheckDisc.ShowNoDeviceErrorMessage(Me)
      Exit Sub
    End If

    ' check number of tracks
    If m_AudioLayoutManager.Tracks.Length = 0 Then
      MessageBox.Show(RM.GetString("Cannot burn audio disc, you didn't add any tracks."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      Return
    End If

    ' Check for too many tracks
    If m_AudioLayoutManager.Tracks.Length > 99 Then
      MessageBox.Show(RM.GetString("The compilation contains more than 99 tracks which is the maximum number of tracks for an audio cd. You will need to remove some tracks before burning this compilation."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      Return
    End If

    ' Player can't be turned off during burning process
    m_AudioPlayer.StopAudio()

    Using f As New frmAudioCDBurn(cboDrives.DeviceSelect.SelectedStarBurnDevice, mAudioImage, m_AudioLayoutManager)
      f.NumberOfCopies = CInt(tbNumCopies.NumericUpDown.Value)
      Me.SwitchToWindow(f)

      BeginRecalcCompilationSize(True)
    End Using

  End Sub

  Private Sub mnuBurnDisc_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuBurnDisc.Click
    tbMidBurn.PerformClick()
  End Sub

  Private Sub mnuEraseDisc_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuEraseDisc.Click
    Using f As New EraseDiscDialog(If(mDevice Is Nothing, Nothing, mDevice.DriveInfo.Letter), Nothing, False)
      f.ShowDialog(Me)
    End Using

    BeginRecalcCompilationSize(True)
  End Sub

  Private Sub LoadCompilation(ByVal strFileName As String)
    If String.IsNullOrEmpty(strFileName) Then Exit Sub

    Try
      m_AudioLayoutManager.LoadAudioCompilation(strFileName)
    Catch ex As Exception
      MessageBox.Show(RM.GetString("An error occured while loading the file:" & " ") & ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
    End Try
  End Sub

  Public Sub SaveCompilation(ByVal filename As String)
    Try
      m_AudioLayoutManager.SaveAudioCompilation(filename)
    Catch ex As Exception
      MessageBox.Show(RM.GetString("An error occured while saving the file:" + " ") & ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
    End Try
  End Sub

  Private Sub m_AudioLayoutManager_CompilationChanged(ByVal sender As System.Object, ByVal e As FileLayoutManager.CompilationChangedEventArgs) Handles m_AudioLayoutManager.CompilationChanged
    Dim strExtraInfo As String = m_AudioLayoutManager.LayoutName

    If m_AudioLayoutManager.IsUnsaved Then
      strExtraInfo = strExtraInfo & "*"
    End If
    Me.Text = strExtraInfo & " - " & RM.GetString("Audio Compilation")

    BeginRecalcCompilationSize(False)

  End Sub

  Private Sub mnuOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuOpen.Click
    If m_AudioLayoutManager.IsUnsaved And m_AudioLayoutManager.Tracks.Length > 0 Then
      If MessageBox.Show(Me, RM.GetString("Opening an audio compilation will clear the current compilation. Do you still want to open a saved compilation"), Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1) = DialogResult.No Then
        Exit Sub
      End If
    End If

    With OpenCompilationDialog
      .Title = RM.GetString("Open Audio Compilation")
      .Filter = RM.GetString("CDBurnerXP Audio Compilations") & " (*.axp)|*.axp|" & RM.GetString("All files") & " (*.*)|*.*"
      .InitialDirectory = ApplicationSettings.AudioSaveCompilationPath
      If .ShowDialog = DialogResult.Cancel Then Exit Sub
      ApplicationSettings.AudioSaveCompilationPath = .FileName
      LoadCompilation(.FileName)
    End With

  End Sub

  Private Sub mnuSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuSave.Click
    If String.IsNullOrEmpty(m_AudioLayoutManager.SavePath) Then
      mnuSaveAs.PerformClick()
    Else
      SaveCompilation(m_AudioLayoutManager.SavePath)
    End If
  End Sub

  Private Sub mnuSaveAs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuSaveAs.Click
    If m_AudioLayoutManager.Tracks.Length = 0 Then
      MessageBox.Show(Me, RM.GetString("Compilation is empty there is nothing to save."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1)
      Exit Sub
    End If

    With SaveCompilationDialog
      .InitialDirectory = ApplicationSettings.AudioSaveCompilationPath
      .Title = RM.GetString("Save Audio Compilation As...")
      .Filter = RM.GetString("CDBurnerXP Audio Compilations") & " (*.axp)|*.axp|" & RM.GetString("Cue sheet") & " (*.cue)|*.cue|" & RM.GetString("M3U playlist") & " (*.m3u)|*.m3u|" & RM.GetString("All files") & " (*.*)|*.*"
      If .ShowDialog(Me) = DialogResult.Cancel Then Exit Sub
      ApplicationSettings.AudioSaveCompilationPath = .FileName
      SaveCompilation(.FileName)
    End With
  End Sub


  Private Sub tbMidBurn_ButtonClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tbMidBurn.Click
    BurnAudioCD()
  End Sub

  Private Sub tbMidErase_ButtonClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tbMidErase.Click
    Using f As New EraseDiscDialog(If(mDevice Is Nothing, Nothing, mDevice.DriveInfo.Letter), Nothing, False)
      f.ShowDialog(Me)
    End Using

    BeginRecalcCompilationSize(True)
  End Sub

  Private Sub tbMidClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tbMidClear.Click
    mnuClearCompilation.PerformClick()
  End Sub

  Friend Sub AddToCompilation(ByVal paths() As String, ByVal pos As Integer)
    m_AudioLayoutManager.BeginAddPaths(paths, pos, MsgBoxOwner)
  End Sub

  Private Sub tbMidAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tbMidAdd.Click
    If filSelect.SelectedCount = 0 Then
      ' user wants to add a file, but didn't select any. we need to help.
      Dim dialog As New OpenFileDialog
      dialog.Multiselect = True
      dialog.InitialDirectory = filSelect.CurrentFolder
      Try
        dialog.Filter = RM.GetString("Music files") & "|" & String.Join(";", Playlists.GetSupportedExtensions(True).ToArray())
      Catch ex As ArgumentException
        ' Bad translation
        dialog.Filter = "Music files|" & String.Join(";", Playlists.GetSupportedExtensions(True).ToArray())
      End Try

      If SafeOpenFileDialog.ShowDialog(dialog, Me) = DialogResult.OK Then
        AddToCompilation(dialog.FileNames, -1)
      End If
      Exit Sub
    End If

    Dim selectedFiles As New List(Of String)
    For Each selectedFile As LogicNP.FileViewControl.ListItem In filSelect.SelectedItems
      selectedFiles.Add(selectedFile.Path)
    Next

    AddToCompilation(selectedFiles.ToArray(), -1)
  End Sub

  Private Sub tbMidRemove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tbMidRemove.Click
    olvTracks.RemoveSelectedTracks()
  End Sub

  Private Sub mnuEject_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuEject.Click
    If mDevice Is Nothing Then Exit Sub
    mDevice.Eject()
  End Sub

  Private Sub mnuCloseTray_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuCloseTray.Click
    LoadDisc(mDevice)
  End Sub

  Private Sub mnuHelpOnLine_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuHelpOnLine.Click
    Try
      System.Diagnostics.Process.Start("http://www.cdburnerxp.se/help/")
    Catch
    End Try
  End Sub

  Private Sub mnuCheckVersion_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuCheckVersion.Click
    AutoUpdate.CheckUpdateOnWebsite()
  End Sub

  Private Sub mnuWeb_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuWeb.Click
    Using oFrmWeb As New frmWeb
      oFrmWeb.ShowDialog(Me)
    End Using
  End Sub

  Private Sub mnuAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuAbout.Click
    Using oFrmAbout As New frmAbout
      oFrmAbout.ShowDialog(Me)
    End Using
  End Sub

  Private Sub mnuDiscInfo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuDiscInfo.Click
    Using f As New DiscInfoDialog()
      f.DriveLetter = If(mDevice Is Nothing, Nothing, mDevice.DriveInfo.Letter)
      f.ShowDialog(Me)
    End Using
  End Sub
  Private Sub mnuOptions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuOptions.Click
    Using oFrmOptions As New frmOptions()
      oFrmOptions.ShowDialog(Me)
    End Using
  End Sub

  Private Sub mnuClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuClose.Click
    m_ExitApplicationAfterClose = False
    Me.Close()
  End Sub

  Private Sub mnuAudioDisc_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuAudioDisc.Click
    Using openDisc As New FolderBrowserDialog()
      openDisc.RootFolder = Environment.SpecialFolder.MyComputer
      openDisc.Description = RM.GetString("Select the drive which contains an audio disc.")
      openDisc.ShowNewFolderButton = False
      If openDisc.ShowDialog(Me) = System.Windows.Forms.DialogResult.OK Then
        m_AudioLayoutManager.BeginAddPaths(New String() {openDisc.SelectedPath}, -1, Me)
      End If
    End Using
  End Sub

  Private Sub mnuCueSheet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuCueSheet.Click
    Using openCue As New OpenFileDialog()
      openCue.InitialDirectory = Me.filSelect.CurrentFolder
      openCue.Filter = RM.GetString("Cue sheets") & " (*.cue)|*.cue"
      If openCue.ShowDialog(Me) = System.Windows.Forms.DialogResult.OK Then
        m_AudioLayoutManager.BeginAddPaths(New String() {openCue.FileName}, -1, Me)
      End If
    End Using
  End Sub

  Private Sub mnuPlaylist_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuPlaylist.Click
    Using openPlaylist As New OpenFileDialog()
      openPlaylist.InitialDirectory = Me.filSelect.CurrentFolder
      openPlaylist.Filter = RM.GetString("Playlists") & " (*.m3u, *.m3u8, *.wpl, *.nra)|*.wpl;*.m3u;*.m3u8;*.nra"
      If openPlaylist.ShowDialog(Me) = System.Windows.Forms.DialogResult.OK Then
        m_AudioLayoutManager.BeginAddPaths(New String() {openPlaylist.FileName}, -1, Me)
      End If
    End Using
  End Sub

#Region "Context menu"

  Private Sub cmnuRandomize_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmnuRandomizeTracks.Click
    m_AudioLayoutManager.RandomizeTracks()
  End Sub

  Private Sub cmnuTracksClearCompilation_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuTracksClearCompilation.Click
    mnuClearCompilation.PerformClick()
  End Sub

  Private Sub cmnuCDText_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuCDText.Click
    mnuCDText.PerformClick()
  End Sub

  Private Sub cmnuMoveUp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuTracksMoveUp.Click
    olvTracks.MoveSelectedTracks(True)
  End Sub

  Private Sub cmnuMoveDown_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuTracksMoveDown.Click
    olvTracks.MoveSelectedTracks(False)
  End Sub

  Private Sub cmnuRemove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuTracksRemoveSelected.Click
    olvTracks.RemoveSelectedTracks()
  End Sub

  Private Sub cmnuSelectAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuTracksSelectAll.Click
    olvTracks.SelectAll()
  End Sub

  Private Sub cmnuPlayTrack_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuTracksPlayTrack.Click
    m_AudioPlayer.PlayAudio(olvTracks.GetSelectedTrackPaths())
    ShowAudioPlayer()
  End Sub

  Private Sub olvTracks_KeyDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles olvTracks.KeyDown
    If e.KeyData = Keys.Enter AndAlso olvTracks.SelectedIndices.Count = 1 Then
      m_AudioPlayer.PlayAudio(CType(olvTracks.SelectedObject, AudioTrack).FileName)
    End If
  End Sub

  Private Sub cmnuTracks_Popup(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuTracks.Popup
    Dim enableItems As Boolean = (olvTracks.SelectedIndices.Count > 0)
    cmnuTracksMoveDown.Enabled = enableItems
    cmnuRandomizeTracks.Enabled = m_AudioLayoutManager.Tracks.Length > 1
    cmnuTracksMoveUp.Enabled = enableItems
    cmnuCDText.Enabled = enableItems
    cmnuTracksRemoveSelected.Enabled = enableItems
    cmnuTracksPlayTrack.Enabled = enableItems
    cmnuTracksPaste.Enabled = olvTracks.CanPaste()
  End Sub

  Private Sub cmnuPaste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuTracksPaste.Click
    olvTracks.Paste()
  End Sub

  Private Sub mnuClearCompilation_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuClearCompilation.Click
    If TaskDialog.ShowOptionalMessageBox(Me, "ClearAudio", RM.GetString("&Clear compilation"), "", RM.GetString("Do you really want to clear the current compilation?"), eTaskDialogButtons.YesNo, eSysIcons.Warning, System.Windows.Forms.DialogResult.Yes) = System.Windows.Forms.DialogResult.Yes Then
      m_AudioLayoutManager.Clear()
    End If
  End Sub

#End Region

  Private Sub mnuDropBox_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuDropBox.Click
    Using f As New frmAudioDropBox(mDevice, Me, m_AudioLayoutManager)
      Me.Visible = False
      MsgBoxOwner = f
      f.ShowDialog(Me)
      Me.Visible = True
    End Using
  End Sub

  Private Sub mnuPrintCover_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuPrintCover.Click
    Using f As New frmPrintCover(RM.GetString("Audio Disc"))
      f.AllowAudioDisplayFormats = True

      Dim albums As New List(Of String)

      For Each item As AudioTrack In m_AudioLayoutManager.AudioTracks
        If Not albums.Contains(item.Album) AndAlso Not String.IsNullOrEmpty(item.Album) Then
          albums.Add(item.Album)
        End If
        f.CoverPrintDocument.FilesAndFolders.Add(New CDBurnerXP.Printing.CoverListItem(item))
      Next

      If albums.Count = 1 Then
        f.txtTitle.Text = albums(0)
      End If

      f.ShowDialog(Me)
    End Using
  End Sub

  Private Sub mnuPrintTrackList_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuPrintTrackList.Click
    Using printer As New ListViewPrinter
      printer.ListView = olvTracks
      printer.DefaultPageSettings.Margins = New System.Drawing.Printing.Margins(20, 20, 20, 20)
      Try
        printer.PrintWithDialog(Me)
      Catch ex As Exception
        MessageBox.Show(Me, ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      End Try
    End Using
  End Sub

  Private Sub mnuPrintTrackListPreview_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuPrintTrackListPreview.Click
    Using printer As New ListViewPrinter
      printer.ListView = olvTracks
      printer.DefaultPageSettings.Margins = New System.Drawing.Printing.Margins(20, 20, 20, 20)
      Try
        printer.PrintPreview(Me, My.Resources.Printer, Me.RightToLeftLayout)
      Catch ex As Exception
        MessageBox.Show(Me, ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      End Try
    End Using
  End Sub

  Private Sub AdjustSplitters()
    Try
      If ApplicationSettings.AudioSplitterVertical > 0 AndAlso ApplicationSettings.AudioSplitterExplorer > 0 Then
        splVertical.SplitterDistance = ApplicationSettings.AudioSplitterVertical
        splExplorer.SplitterDistance = ApplicationSettings.AudioSplitterExplorer
      Else
        If splVertical.Orientation = Orientation.Horizontal Then
          splVertical.SplitterDistance = CInt(Me.Height * 0.4)
          splExplorer.SplitterDistance = CInt(Me.Width * 0.3)
        Else
          splVertical.SplitterDistance = CInt(Me.Width * 0.45)
          splExplorer.SplitterDistance = CInt(Me.Width * 0.2)
        End If
      End If
    Catch ex As InvalidOperationException
      ' ignore if it fails...window too small, resize too early.
      ' All rare and can't do much about it.
    End Try
  End Sub

  Private Sub mnuExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuExit.Click
    Me.Close()
  End Sub

  Private Sub mnuCDText_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuCDText.Click
    Using dialog As New CDTextDialog
      dialog.LayoutManager = Me.m_AudioLayoutManager
      dialog.TrackNumber = Me.m_AudioLayoutManager.ListView.SelectedIndex + 1
      dialog.ShowDialog(Me)
    End Using
  End Sub

  Private Sub lvwTracks_DoubleClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles olvTracks.DoubleClick
    If olvTracks.SelectedIndices.Count = 1 Then
      m_AudioPlayer.PlayAudio(CType(olvTracks.SelectedObject, AudioTrack).FileName)
      ShowAudioPlayer()
    End If
  End Sub

  Private Sub mnuAddFiles_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuAddFiles.Click
    tbMidAdd.PerformClick()
  End Sub

  Private Sub mnuFileFilter_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileFilter.Click
    Using f As New frmFilter(False)
      If f.ShowDialog(Me) = DialogResult.OK Then
        filSelect.RefreshView()
      End If
    End Using
  End Sub

  Private Sub mnuSelectAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuSelectAll.Click
    If olvTracks.Focused Then
      cmnuTracksSelectAll.PerformClick()
    Else
      Dim lstItm As LogicNP.FileViewControl.ListItem
      For i As Integer = 1 To filSelect.ItemCount
        lstItm = filSelect.GetListItem(i)
        lstItm.Selected = True
      Next
    End If
  End Sub

  Private Sub mnuAddFolder_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuAddFolder.Click
    Using dialog As New FolderBrowserDialog
      dialog.ShowNewFolderButton = False

      Dim lastFolder As String = CStr(Settings.GetValue("Paths", "AudioAddFolder", ""))
      If String.IsNullOrEmpty(lastFolder) Then
        dialog.SelectedPath = filSelect.CurrentFolder
      Else
        dialog.SelectedPath = lastFolder
      End If

      If dialog.ShowDialog(Me) = System.Windows.Forms.DialogResult.OK Then
        Settings.SetValue("Paths", "AudioAddFolder", dialog.SelectedPath)
        AddToCompilation(New String() {dialog.SelectedPath}, -1)
      End If
      Exit Sub
    End Using
  End Sub

  Private Sub mnuInvertSelection_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuInvertSelection.Click
    If olvTracks.Focused Then
      olvTracks.InvertSelection()
    Else
      filSelect.InvertSelection()
    End If
  End Sub

  Private Sub filSelect_ItemDblClick(ByVal sender As System.Object, ByVal e As LogicNP.FileViewControl.FileViewCancelEventArgs) Handles filSelect.ItemDblClick
    If Not e.Item.IsFolder AndAlso Playlists.IsAudioFile(e.Item.Path) Then
      m_AudioPlayer.PlayAudio(e.Item.Path)
      e.Cancel = True
    End If
  End Sub

  Private Sub mnuRecorderInfo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuRecorderInfo.Click
    'Recorder Information
    Using frmRecInfo As New frmRecorderInfo(BurningEngine.GetDevice(mDevice))
      frmRecInfo.ShowDialog(Me)
    End Using
  End Sub

  Private Sub mnuReportDrive_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuReportDrive.Click
    Using f As New frmReportDrive()
      f.ShowDialog(Me)
    End Using
  End Sub

#Region "Sizebar context menu"

  Private Sub cmnuSizebarAutoDetect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarAutoDetect.Click
    m_ForceMedia = 0
    BeginRecalcCompilationSize(False)
  End Sub

  Private Sub cmnuCD21_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarCD21.Click
    m_ForceMedia = 21 * 60
    BeginRecalcCompilationSize(False)
  End Sub

  Private Sub cmnuCD63_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarCD63.Click
    m_ForceMedia = 63 * 60
    BeginRecalcCompilationSize(False)
  End Sub

  Private Sub cmnuCD74_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarCD74.Click
    m_ForceMedia = 74 * 60
    BeginRecalcCompilationSize(False)
  End Sub

  Private Sub cmnuCD80_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarCD80.Click
    m_ForceMedia = 80 * 60
    BeginRecalcCompilationSize(False)
  End Sub

  Private Sub cmnuCD90_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarCD90.Click
    m_ForceMedia = 90 * 60
    BeginRecalcCompilationSize(False)
  End Sub

  Private Sub cmnuSizebarRefreshDisc_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarRefreshDisc.Click
    BeginRecalcCompilationSize(True)
  End Sub

#End Region

End Class