﻿using System;
using System.Collections.Generic;
using System.Text;
using NMSDVDXLib;
using System.IO;

namespace CDBurnerXP.Burning
{
    /// <summary>
    /// Contains information about a file or file, which has to be 
    /// renamed on the disc due to file system limitations.
    /// </summary>
    public class FileRenameRequirement
    {
        public string originalName;
        public string newName;
        public string pathOnDisc;
        public bool isFolder = false;

        public FileRenameRequirement(IDataItemX dataItem, string newName)
        {
            this.originalName = dataItem.GetImageName();
            this.isFolder = dataItem.IsFolder();
            this.newName = newName;
            this.pathOnDisc = dataItem.GetImagePath();
            //Slow: this.pathOnDisc = GetPathOnDisc(dataItem);
        }


        /// <summary>
        /// Determines the path of an item on the disc (without
        /// uppercasing the path).
        /// </summary>
        private static string GetPathOnDisc(IDataItemX item)
        {
            List<string> parts = new List<string>();
            IDataItemX parent = item.GetParentFolder();
            while (parent != null)
            {
                parts.Add(parent.GetImageName());
                parent = parent.GetParentFolder();
            }
            parts.Reverse();
            return Path.Combine(string.Join("\\", parts.ToArray()).TrimStart('\\'), item.GetImageName());
        }
    }
}
