using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32;
using CDBurnerXP.Forms;

namespace CDBurnerXP.Controls
{
    /// <summary>
    /// The following modifications apply:
    ///  - Flicker free
    ///  - Selection does not jump when using the right mouse button
    ///  - Copy and paste works during label edit
    /// </summary>
    public class TreeViewEx : TreeView, IMirrorControl
    {
        private IntPtr m_EditBox = IntPtr.Zero;

        [System.Diagnostics.DebuggerStepThrough()]
        protected override void WndProc(ref Message message)
        {
            switch (message.Msg)
            {
                case User32.WM_ERASEBKGND:
                    //removes flicker
                    return;

            }

            base.WndProc(ref message);
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                TreeNode selection = GetNodeAt(e.Location);
                if (selection != null)
                {
                    SelectedNode = selection;
                }
            }
            base.OnMouseDown(e);
        }

        protected override void OnBeforeLabelEdit(NodeLabelEditEventArgs e)
        {
            base.OnBeforeLabelEdit(e);
            m_EditBox = User32.FindWindowEx(this.Handle, IntPtr.Zero, "Edit", null);
        }

        protected override void OnAfterLabelEdit(NodeLabelEditEventArgs e)
        {
            base.OnAfterLabelEdit(e);
            m_EditBox = IntPtr.Zero;
        }

        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            if (m_EditBox != IntPtr.Zero)
            {
                // Fix Copy and Paste
                switch (keyData)
                {
                    case Keys.Control | Keys.C:
                        User32.SendMessage(m_EditBox, User32.WM_COPY, IntPtr.Zero, IntPtr.Zero);
                        return true;

                    case Keys.Control | Keys.V:
                        User32.SendMessage(m_EditBox, User32.WM_PASTE, IntPtr.Zero, IntPtr.Zero);
                        return true;

                    case Keys.Control | Keys.X:
                        User32.SendMessage(m_EditBox, User32.WM_CUT, IntPtr.Zero, IntPtr.Zero);
                        return true;
                }
            }
            return base.ProcessCmdKey(ref msg, keyData);
        }

        #region IMirrorControl Member

        public bool Mirror { get; set; }

        protected override CreateParams CreateParams
        {
            get
            {
                if (this.Mirror || BaseForm.IsRightToLeftCulture)
                {
                    CreateParams mirroredCp = base.CreateParams;
                    mirroredCp.ExStyle = mirroredCp.ExStyle | 0x400000;
                    return mirroredCp;
                }

                return base.CreateParams;
            }
        }

        #endregion
    }
}
