﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CDBurnerXP.Forms;
using System.IO;
using RocketDivision.StarBurnX;
using System.Threading;
using System.Runtime.InteropServices;
using CDBurnerXP.Burning;
using CDBurnerXP.Controls;
using Microsoft.Win32;
using CDBurnerXP.Windows.Forms;
using CDBurnerXP.IO;

namespace CDBurnerXP.Forms
{
    public partial class BurnVideoDvdDialog : BurningBaseForm
    {
        private static ImageList animations = null;
        private DVDVideoBurner videoDvdBurner = null;
        private string burnSpeedText = string.Empty;
        private bool cancelled = false;

        #region Properties

        protected override bool AutoTranslate
        {
            get
            {
                return true;
            }
        }

        protected override bool UseRemainingCalculator
        {
            get
            {
                return false;
            }
        }

        public override int NumberOfCopies
        {
            get
            {
                return Convert.ToInt32(numCopies.Value);
            }
            set
            {
                base.NumberOfCopies = value;
                numCopies.Value = Convert.ToInt32(value > 0 ? value : 1);
            }
        }

        protected override RocketDivision.StarBurnX.Drive StarBurnDevice
        {
            get
            {
                return cbDevice.SelectedStarBurnDevice;
            }
        }

        protected string VideoTSFolder
        {
            get
            {
                return this.txtVideoTS.Text.Trim(' ', '\t', '\r', '\n', '"');
            }
        }

        public static ImageList Animations
        {
            get { return BurnVideoDvdDialog.animations; }
            set { BurnVideoDvdDialog.animations = value; }
        }

        #endregion

        public BurnVideoDvdDialog()
        {
            InitializeComponent();

            AcceptButton = cmdStartBurn;
            CancelButton = cmdCancel;

            cbDevice.SelectedDeviceLetter = ApplicationSettings.VideoLastDeviceUsed;
            cbDevice.SelectedIndexChanged += new EventHandler(cbDevice_SelectedIndexChanged);
            cbDevice_SelectedIndexChanged(this, EventArgs.Empty);
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            if (DesignMode) return;

            sepSettings.Text = RM.GetString("Burn Options");
            tabBurnOptions.Text = RM.GetString("Burn Options|Tab");
            tabBurnProgress.Text = RM.GetString("Burn Progress");
            chkEject.Text = RM.GetString("Eject Disc After Burn");
            cmdStartBurn.Text = RM.GetString("Start Burn");
            cmdCancel.Text = RM.GetString("Cancel");
            lblStatus.Text = RM.GetString("Status: Idle");
            lblTime.Text = String.Format(RM.GetString("Time Elapsed (hh:mm:ss): {0}"), "");
            lblRemaining.Text = String.Format(RM.GetString("Time Remaining (hh:mm:ss): {0}"), "");
            chkShutdown.Text = RM.GetString("Shutdown computer after completion");
            lblCopies.Text = RM.GetString("&Number of copies:");
            toolTip.ToolTipTitle = chkPadSize.Text.Replace("&", "");
            toolTip.SetToolTip(chkPadSize, RM.GetString("To ensure compatibility with all DVD players, fill the DVD with 1GB of data,\r\neven if the total size of the video files is lower than 1GB."));
            m_AutoImportEnabled = false;
            Animation.ImageList = Animations;
            Animation.ShowFrame(0);

            OptimizeSize(this.bSaveAsIso);
            this.bSaveAsIso.Left = this.cmdCancel.Left - this.bSaveAsIso.Width - 3;
            OptimizeSize(this.cmdStartBurn);
            this.cmdStartBurn.Left = this.bSaveAsIso.Left - this.cmdStartBurn.Width - 3;

            chkEject.Checked = ApplicationSettings.DataBurnEject;
            chkDoubleLayerCompatible.Checked = ApplicationSettings.DoubleLayerCompatible;
            chkPadSize.Checked = ApplicationSettings.PadDvdSize;
            chkModifyFiles.Checked = ApplicationSettings.ModifyIfoBupFiles;

            // Set last used burn speed, if present
            cdbSpeed.SetLastWriteSpeed = ApplicationSettings.LastDataBurnSpeed;
            ActivateControls(true);
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            base.OnClosing(e);

            if (m_Status != Status.Idle)
            {
                e.Cancel = true;
                return;
            }

            ApplicationSettings.DataBurnEject = chkEject.Checked;
            ApplicationSettings.LastDataBurnSpeed = cdbSpeed.WriteSpeedLongValue;
            ApplicationSettings.DoubleLayerCompatible = chkDoubleLayerCompatible.Checked;
            ApplicationSettings.PadDvdSize = chkPadSize.Checked;
            ApplicationSettings.ModifyIfoBupFiles = chkModifyFiles.Checked;
            ApplicationSettings.VideoLastDeviceUsed = cbDevice.SelectedDeviceLetter;

            m_AutoImportEnabled = true;
        }

        protected override void OnDragEnter(DragEventArgs drgevent)
        {
            base.OnDragEnter(drgevent);

            CheckDragDrop(drgevent);
        }

        protected override void OnDragOver(DragEventArgs drgevent)
        {
            base.OnDragOver(drgevent);

            CheckDragDrop(drgevent);
        }

        private void CheckDragDrop(DragEventArgs drgevent)
        {
            if (drgevent.Data.GetDataPresent(DataFormats.FileDrop))
            {
                drgevent.Effect = DragDropEffects.Copy;
            }
        }

        protected override void OnDragDrop(DragEventArgs drgevent)
        {
            base.OnDragDrop(drgevent);

            string[] files = drgevent.Data.GetData(DataFormats.FileDrop) as string[];
            if (files.Length > 0)
            {
                string folder = files[0];
                string videoTsSub = Path.Combine(folder, "VIDEO_TS");
                txtVideoTS.Text = Directory.Exists(videoTsSub)? videoTsSub : folder;
            }
        }

        protected override void OnMediaChanged(string VolumeLabel)
        {
            base.OnMediaChanged(VolumeLabel);

            ActivateControls(true);
        }

        protected override void ActivateControls(bool enable)
        {
            base.ActivateControls(enable);

            cdbSpeed.Enabled = enable;
            foreach (Control control in tabBurnOptions.Controls)
            {
                CheckBox checkBox = control as CheckBox;
                if (checkBox != null)
                {
                    checkBox.Enabled = enable;
                }
            }

            txtVideoTS.Enabled = enable;
            txtDiscName.Enabled = enable;
            lblDiscName.Enabled = enable;
            lblVideoTS.Enabled = enable;
            bSelectFolder.Enabled = enable;
            cbDevice.Enabled = enable;
            lblSourceDevice.Enabled = enable;

            chkEject.Enabled = true;
            chkShutdown.Enabled = true;
            cmdStartBurn.Enabled = enable && !string.IsNullOrEmpty(this.VideoTSFolder);
            bSaveAsIso.Enabled = enable && !string.IsNullOrEmpty(this.VideoTSFolder);
        }

        private void txtVideoTS_TextChanged(object sender, EventArgs e)
        {
            cmdStartBurn.Enabled = !string.IsNullOrEmpty(this.VideoTSFolder);
            bSaveAsIso.Enabled = !string.IsNullOrEmpty(this.VideoTSFolder);
        }

        private void cbDevice_SelectedIndexChanged(object sender, EventArgs e)
        {
            cdbSpeed.BurnSpeedDeviceLetter = cbDevice.SelectedDeviceLetter;
        }

        private void bSaveAsIso_Click(object sender, EventArgs e)
        {
            long fileSize;
            if (!TryGetFileSize(out fileSize))
            {
                return;
            }

            using (SaveFileDialog dialog = new SaveFileDialog())
            {
                dialog.Title = RM.GetString("Save As ISO File");
                dialog.InitialDirectory = ApplicationSettings.DataSaveISOPath;
                dialog.FileName = txtDiscName.Text + ".iso";
                dialog.Filter = RM.GetString("ISO File") + " (*.iso)|*.iso";

                if (dialog.ShowDialog(this) == DialogResult.OK)
                {
                    if (!DiskSpaceRequiredDialog.Show(this, fileSize, dialog.FileName))
                    {
                        // not an error, user must have clicked "cancel"
                        return;
                    }

                    CreateVideoBurner();

                    using (ProgressDialog progressDialog = new ProgressDialog("Creating ISO Image", "Creating ISO image, please wait..."))
                    {
                        // Attach necessary watching events
                        IDvdVideoBurnerEvents_OnProgressEventHandler statusDelegate = delegate(int percent, int timeRemaining)
                        {
                            progressDialog.ReportProgress(percent);
                        };

                        string volumeName = txtDiscName.Text;
                        this.videoDvdBurner.OnProgress += statusDelegate;

                        // Actual operation
                        try
                        {
                            progressDialog.OnCancel = delegate() { this.videoDvdBurner.Cancel(); };
                            progressDialog.OnDoWork = delegate()
                            {
                                this.videoDvdBurner.SaveImageFile(dialog.FileName, volumeName, string.Empty, string.Empty);
                                return true;
                            };
                            progressDialog.ShowDialog(this);
                        }
                        finally
                        {
                            this.videoDvdBurner.OnProgress -= statusDelegate;
                        }
                    }
                }
            }
        }

        private void bSelectFolder_Click(object sender, EventArgs e)
        {
            using (SelectVideoFolderDialog dialog = new SelectVideoFolderDialog())
            {
                if (dialog.ShowDialog(this) == DialogResult.OK)
                {
                    txtVideoTS.Text = dialog.SelectedPath;

                    // Suggest disc name by folder name
                    if (string.IsNullOrEmpty(this.txtDiscName.Text))
                    {
                        try
                        {
                            txtDiscName.Text = Conversion.Limit(Path.GetFileName(Path.GetDirectoryName(dialog.SelectedPath)), 32);
                        }
                        catch (Exception)
                        {
                        }
                    }
                }
            }
        }

        #region Burn handling

        protected override void OnStartBurnClick(object sender, EventArgs e)
        {
            long fileSize;
            if (!TryGetFileSize(out fileSize))
            {
                return;
            }

            base.OnStartBurnClick(sender, e);

            if (!CheckDisc.Show(this, StarBurnDevice, CheckDisc.DiscFeature.Writable, CheckDisc.DiscType.DVD, fileSize, false, false))
            {
                ResetUiAfterWork();
                return;
            }

            m_StartTime = DateTime.Now;
            tmrElapsed.Enabled = true;

            CreateVideoBurner();
            this.videoDvdBurner.OnProgress += new IDvdVideoBurnerEvents_OnProgressEventHandler(DvdBurnerOnProgress);
            this.videoDvdBurner.OnCloseSessionBegin += new IDvdVideoBurnerEvents_OnCloseSessionBeginEventHandler(DvdBurnerOnCloseSessionBegin);

            // Make sure that new speeds are loaded, after CheckDisc ensured that disc has been inserted
            cdbSpeed.UpdateBurningSpeeds();

            this.videoDvdBurner.WriteSpeed = cdbSpeed.WriteSpeedLongValue;
            this.burnSpeedText = cdbSpeed.WriteSpeedDescription;

            PrepareUiForWork();
            m_Status = Status.Burning;

            Thread burningThread = new Thread(new ParameterizedThreadStart(StartBurnThread));
            burningThread.Start(txtDiscName.Text);
        }

        private void CreateVideoBurner()
        {
            this.videoDvdBurner = new DVDVideoBurner();
            this.videoDvdBurner.NextSessionAllowed = false;
            this.videoDvdBurner.DoubleLayerCompatibleMode = chkDoubleLayerCompatible.Checked;
            this.videoDvdBurner.Drive = StarBurnDevice;
            this.videoDvdBurner.Padding = chkPadSize.Checked;
            this.videoDvdBurner.PatchingIFOBUP = chkModifyFiles.Checked;
            this.videoDvdBurner.VideoDirectory = this.VideoTSFolder;
        }

        private bool TryGetFileSize(out long fileSize)
        {
            // Determine size of all files
            string videoPath = VideoTSFolder;
            fileSize = 0;
            try
            {
                foreach (string file in Directory.GetFiles(videoPath, "*", SearchOption.AllDirectories))
                {
                    fileSize += PathEx.TryGetFileSize(file);
                }

                return true;
            }
            catch (NotSupportedException)
            {
                MessageBox.Show(this, FormatTranslation("The data source \"{0}\" is not supported.\r\n\r\nPlease select a folder from your hard disk or a network location.", videoPath), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            catch (PathTooLongException)
            {
                MessageBox.Show(this, FormatTranslation("The directory \"{0}\" cannot be found.\r\n\r\nPlease make sure that the directory exists, or select another directory.", videoPath), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            catch (DirectoryNotFoundException)
            {
                MessageBox.Show(this, FormatTranslation("The directory \"{0}\" cannot be found.\r\n\r\nPlease make sure that the directory exists, or select another directory.", videoPath), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            catch (ArgumentException)
            {
                // Invalid path chars
                MessageBox.Show(this, FormatTranslation("The directory \"{0}\" cannot be found.\r\n\r\nPlease make sure that the directory exists, or select another directory.", videoPath), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            catch (IOException)
            {
                MessageBox.Show(this, FormatTranslation("The directory \"{0}\" cannot be accessed.\r\n\r\nPlease make sure that the directory exists and you have permission to read the directory.", videoPath), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            catch (UnauthorizedAccessException)
            {
                MessageBox.Show(this, FormatTranslation("The directory \"{0}\" cannot be accessed.\r\n\r\nPlease make sure that the directory exists and you have permission to read the directory.", videoPath), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
        }

        protected override void OnCancelClick(object sender, EventArgs e)
        {
            base.OnCancelClick(sender, e);

            if (m_Status == Status.Idle)
            {
                this.Close();
                return;
            }

            if (MessageBox.Show(this, RM.GetString("Are you sure that you want to cancel the current burning process?"), Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
            {
                cmdCancel.Tag = null;
                lblStatus.Text = RM.GetString("Status: Aborting burn, please wait...");
                lvwStatus.AddItem(LogListView.LogType.Warning, RM.GetString("Burning was cancelled by the user, aborting please wait"));
                this.cancelled = true;
                this.videoDvdBurner.Cancel();
            }
        }

        protected override void SetBurnProcessStartedText()
        {
            lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Burn process started at {0}"), this.burnSpeedText));
            lblStatus.Text = RM.GetString("Preparing to write disc...");
        }

        protected override void ResetUiAfterWork()
        {
            base.ResetUiAfterWork();
            this.Text = RM.GetString("Burn Video DVD");
        }

        private void StartBurnThread(object discName)
        {
            try
            {
                this.cancelled = false;
                HandleBurnProcessStarted();

                this.videoDvdBurner.Burn(false, discName as string, string.Empty, string.Empty);
            }
            catch (COMException ex)
            {
                this.Invoke((MethodInvoker)delegate { HandleError(ex); });
            }
            finally
            {
                this.BeginInvoke((MethodInvoker)delegate { HandleWriteComplete(); });
                BurningEngine.TryUnlock(this.videoDvdBurner.Drive);
            }
        }

        private void HandleWriteComplete()
        {
            prgProgress.Style = ProgressBarStyle.Blocks;
            prgProgress.Value = 100;
            tmrElapsed.Enabled = false;

            if (this.videoDvdBurner.LastErrorCode == STARBURN_RESULT_CODES.STARBURN_OPERATION_SUCCEEDED)
            {
                lblStatus.Text = RM.GetString("Status: Burning completed...");

                if (Animation.FirstFrame != 50)
                {
                    Animation.FirstFrame = 50;
                    Animation.LastFrame = 61;
                    Animation.Delay = 90;
                    Animation.Pause = 500;
                    Animation.StartAnimation();
                }

                prgProgress.ShowMarquee = false;
                lblStatus.Text = RM.GetString("Status: Idle");
                lblTime.Text = String.Format(RM.GetString("Time Elapsed (hh:mm:ss): {0}"), "");
                prgProgress.Style = ProgressBarStyle.Blocks;

                if (chkEject.Checked)
                {
                    StarBurnDevice.Eject();
                }

                if (!this.cancelled)
                {
                    if (chkShutdown.Checked)
                    {
                        ExitWindows.ForceShutdown();
                        return;
                    }

                    lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Burning process is successfully completed at speed {0}"), this.burnSpeedText));

                    TimeSpan deltaTime = Utility.TimeSpanSeconds(m_ElapsedTime - m_StartTime);
                    bool furtherCopies = FurtherCopiesRequired();

                    if (!furtherCopies)
                    {
                        // Show result message box. If verifying has been cancelled, make not of it (and that not the actual burning process failed).
                        string msg = String.Format(RM.GetString("Burning process is successfully completed at speed {0}"), this.burnSpeedText) + Environment.NewLine + String.Format(RM.GetString("Total time for operation (hh:mm:ss): {0}"), deltaTime.ToString());
                        TaskDialog.ShowOptionalMessageBox(this, "DataCompleted", Application.ProductName, "", msg, eTaskDialogButtons.OK, eSysIcons.Information, 0);
                    }
                    else
                    {
                        BeginBurnNextDisc();
                        return;
                    }
                }

                ResetUiAfterWork();
                prgProgress.Value = 0;
            }
        }

        private void DvdBurnerOnCloseSessionBegin()
        {
        }

        private void DvdBurnerOnProgress(int percent, int timeRemaining)
        {
            prgProgress.Value = percent;

            if (percent > 0)
            {
                if (Animation.FirstFrame != 1)
                {
                    Animation.FirstFrame = 1;
                    Animation.LastFrame = 18;
                    Animation.Delay = 80;
                    Animation.Pause = 50;
                    Animation.StartAnimation();
                }

                lblStatus.Text = RM.GetString("Status: Burning disc...");
            }

            SetTimeRemaining(timeRemaining);
            this.Text = FormatTranslation("{0}% Burning To Media - [{1}]", percent, string.IsNullOrEmpty(txtDiscName.Text) ? RM.GetString("Video DVD") : txtDiscName.Text);
        }

        #endregion
    }
}
