﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CDBurnerXP.Burning;
using CDBurnerXP.IO;

namespace CDBurnerXP.Forms
{
    public partial class CustomDiscSizeDialog : BaseForm
    {
        #region Properties

        protected override bool AutoTranslate
        {
            get
            {
                return true;
            }
        }

        public DiscType CustomDiscType
        {
            get
            {
                switch (cboUnit.SelectedIndex)
                {
                    default:
                    case 0: return new DiscType(FormatTranslation("(Custom: {0})", FormatFileSize.Format((long)numSize.Value)), (long)numSize.Value);
                    case 1: return new DiscType(FormatTranslation("(Custom: {0})", FormatFileSize.Format((long)numSize.Value * 1024)), (long)numSize.Value * 1024);
                    case 2: return new DiscType(FormatTranslation("(Custom: {0})", FormatFileSize.Format((long)numSize.Value * 1024 * 1024)), (long)numSize.Value * 1024 * 1024);
                    case 3: return new DiscType(FormatTranslation("(Custom: {0})", FormatFileSize.Format((long)numSize.Value * 1024 * 1024 * 1024)), (long)numSize.Value * 1024 * 1024 * 1024);
                }
            }
            set
            {
                if (value != null)
                {
                    numSize.Value = value.SizeInBytes;
                    cboUnit.SelectedIndex = 0;
                }
            }
        }

        #endregion

        public CustomDiscSizeDialog()
        {
            InitializeComponent();
            AcceptButton = bOK;
            CancelButton = bCancelStart;

            cboUnit.SelectedIndex = 2;

            colSize.AspectToStringConverter = delegate(object x)
            {
                return FormatFileSize.Format((long)x);
            };

            cboUnit.Items[0] = RM.GetString(cboUnit.Items[0] as string);
            cboUnit.Items[1] = RM.GetString(cboUnit.Items[1] as string);
            cboUnit.Items[2] = RM.GetString(cboUnit.Items[2] as string);
            cboUnit.Items[3] = RM.GetString(cboUnit.Items[3] as string);

            colMedium.Text = RM.GetString("Medium");
            colSize.Text = RM.GetString("Size");
        }

        protected override void OnLoad(EventArgs e)
        {
            if (DesignMode) return;

            base.OnLoad(e);

            List<DiscType> sizes = new List<DiscType>();
            sizes.Add(DiscType.CD21);
            sizes.Add(DiscType.CD63);
            sizes.Add(DiscType.CD74);
            sizes.Add(DiscType.CD80);
            sizes.Add(DiscType.CD90);
            sizes.Add(DiscType.CD99);
            sizes.Add(DiscType.DVD);
            sizes.Add(DiscType.DVDPlus);
            sizes.Add(DiscType.DVDDL);
            sizes.Add(DiscType.HDDVD);
            sizes.Add(DiscType.HDDVDDL);
            sizes.Add(DiscType.BD);
            sizes.Add(DiscType.BDDL);
            sizes.Add(new DiscType("ZIP 100MB", 100 * 1000 * 1000));
            sizes.Add(new DiscType("ZIP 250MB", 250 * 1000 * 1000));
            sizes.Add(new DiscType("ZIP 750MB", 750 * 1000 * 1000));
            sizes.Add(new DiscType("LS120", 126222336));
            sizes.Add(new DiscType("LS240", 126222336*2));
            olvSizes.SetObjects(sizes);

            numSize.Left = lblCustom.Right + 3;
            cboUnit.Left = numSize.Right + 6;
        }

        private void olvSizes_SelectedIndexChanged(object sender, EventArgs e)
        {
            DiscType t = olvSizes.SelectedObject as DiscType;
            if (t != null)
            {
                numSize.Value = t.SizeInBytes;
                cboUnit.SelectedIndex = 0;
            }
        }

        private void olvSizes_DoubleClick(object sender, EventArgs e)
        {
            bOK.PerformClick();
        }

    }
}
