using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Resources;
using System.Drawing;
using CDBurnerXP.Forms;

namespace CDBurnerXP.Windows.Forms
{
    #region PUBLIC enums
    //--------------------------------------------------------------------------------
    public enum eSysIcons { Information, Question, Warning, Error };
    public enum eTaskDialogButtons { YesNo, YesNoCancel, OKCancel, OK, Close, Cancel, None }

    #endregion

    public static class TaskDialog
    {
        public enum UIText
        {
            Cancel,
            Close,
            HideDetails,
            ShowDetails,
            OK,
            Yes,
            No
        }

        // Defines
        private static int EmulatedFormWidth = 450;
        private static Dictionary<UIText, string> m_Translations = new Dictionary<UIText, string>();

        #region Properties

        public static Dictionary<UIText, string> Translation
        {
            get
            {
                return m_Translations;
            }
        }

        #endregion

        //--------------------------------------------------------------------------------
        #region ShowTaskDialogBox
        //--------------------------------------------------------------------------------
        static public TaskDialogResult ShowTaskDialogBox(IWin32Window owner, string Title,
                                                     string MainInstruction,
                                                     string Content,
                                                     string ExpandedInfo,
                                                     string Footer,
                                                     string VerificationText,
                                                     string RadioButtons,
                                                     string CommandButtons,
                                                     eTaskDialogButtons Buttons,
                                                     eSysIcons MainIcon,
                                                     eSysIcons FooterIcon, Icon customIcon)
        {
            if (Title != null) Title = Title.Replace("&", "");

            if (VistaTaskDialog.IsAvailableOnThisOS)
            {
                // [OPTION 1] Show Vista TaskDialog
                VistaTaskDialog vtd = new VistaTaskDialog();

                vtd.WindowTitle = Title;
                vtd.MainInstruction = MainInstruction;
                vtd.Content = Content;
                vtd.ExpandedInformation = ExpandedInfo;
                vtd.Footer = Footer;

                // Radio Buttons
                if (RadioButtons != "")
                {
                    List<VistaTaskDialogButton> lst = new List<VistaTaskDialogButton>();
                    string[] arr = RadioButtons.Split(new char[] { '|' });
                    for (int i = 0; i < arr.Length; i++)
                    {
                        try
                        {
                            VistaTaskDialogButton button = new VistaTaskDialogButton();
                            button.ButtonId = 1000 + i;
                            button.ButtonText = arr[i];
                            lst.Add(button);
                        }
                        catch (FormatException)
                        {
                        }
                    }
                    vtd.RadioButtons = lst.ToArray();
                }

                // Custom Buttons
                if (CommandButtons != "")
                {
                    List<VistaTaskDialogButton> lst = new List<VistaTaskDialogButton>();
                    string[] arr = CommandButtons.Split(new char[] { '|' });
                    for (int i = 0; i < arr.Length; i++)
                    {
                        try
                        {
                            VistaTaskDialogButton button = new VistaTaskDialogButton();
                            button.ButtonId = 2000 + i;
                            button.ButtonText = arr[i];
                            lst.Add(button);
                        }
                        catch (FormatException)
                        {
                        }
                    }
                    vtd.Buttons = lst.ToArray();
                }

                switch (Buttons)
                {
                    case eTaskDialogButtons.YesNo:
                        vtd.CommonButtons = VistaTaskDialogCommonButtons.Yes | VistaTaskDialogCommonButtons.No;
                        break;
                    case eTaskDialogButtons.YesNoCancel:
                        vtd.CommonButtons = VistaTaskDialogCommonButtons.Yes | VistaTaskDialogCommonButtons.No | VistaTaskDialogCommonButtons.Cancel;
                        break;
                    case eTaskDialogButtons.OKCancel:
                        vtd.CommonButtons = VistaTaskDialogCommonButtons.Ok | VistaTaskDialogCommonButtons.Cancel;
                        break;
                    case eTaskDialogButtons.OK:
                        vtd.CommonButtons = VistaTaskDialogCommonButtons.Ok;
                        break;
                    case eTaskDialogButtons.Close:
                        vtd.CommonButtons = VistaTaskDialogCommonButtons.Close;
                        break;
                    case eTaskDialogButtons.Cancel:
                        vtd.CommonButtons = VistaTaskDialogCommonButtons.Cancel;
                        break;
                    default:
                        vtd.CommonButtons = 0;
                        break;
                }

                if (customIcon != null)
                {
                    vtd.CustomMainIcon = customIcon;
                }
                else
                {
                    switch (MainIcon)
                    {
                        case eSysIcons.Information: vtd.MainIcon = VistaTaskDialogIcon.Information; break;
                        case eSysIcons.Question: vtd.MainIcon = VistaTaskDialogIcon.Question; break;
                        case eSysIcons.Warning: vtd.MainIcon = VistaTaskDialogIcon.Warning; break;
                        case eSysIcons.Error: vtd.MainIcon = VistaTaskDialogIcon.Error; break;
                    }
                }

                switch (FooterIcon)
                {
                    case eSysIcons.Information: /* Information icon not allowed in footer area. The footer already implies that it is an information! */ break;
                    case eSysIcons.Question: vtd.FooterIcon = VistaTaskDialogIcon.Question; break;
                    case eSysIcons.Warning: vtd.FooterIcon = VistaTaskDialogIcon.Warning; break;
                    case eSysIcons.Error: vtd.FooterIcon = VistaTaskDialogIcon.Error; break;
                }

                vtd.EnableHyperlinks = false;
                vtd.ShowProgressBar = false;
                vtd.AllowDialogCancellation = (Buttons == eTaskDialogButtons.Cancel ||
                                               Buttons == eTaskDialogButtons.Close ||
                                               Buttons == eTaskDialogButtons.OKCancel ||
                                               Buttons == eTaskDialogButtons.YesNoCancel);
                vtd.CallbackTimer = false;
                vtd.ExpandedByDefault = false;
                vtd.ExpandFooterArea = false;
                vtd.PositionRelativeToWindow = true;
                vtd.RightToLeftLayout = false;
                vtd.NoDefaultRadioButton = false;
                vtd.CanBeMinimized = false;
                vtd.ShowMarqueeProgressBar = false;
                vtd.UseCommandLinks = (CommandButtons != "");
                vtd.UseCommandLinksNoIcon = false;
                vtd.VerificationText = VerificationText;
                vtd.VerificationFlagChecked = false;
                vtd.ExpandedControlText = TaskDialog.Translation[TaskDialog.UIText.HideDetails];
                vtd.CollapsedControlText = TaskDialog.Translation[TaskDialog.UIText.ShowDetails];
                vtd.Callback = null;

                // Show the Dialog
                TaskDialogResult result = vtd.Show(owner);

                // if a command button was clicked, then change return result
                // to "DialogResult.OK" and set the CommandButtonResult
                if (result.SelectedCommand >= 2000)
                {
                    result.SelectedCommand -= 2000;
                    result.DialogResult = DialogResult.OK;
                }
                else
                {
                    result.DialogResult = (DialogResult)result.SelectedCommand;
                }
                if (result.SelectedRadioButton >= 1000)
                    result.SelectedRadioButton -= 1000;  // deduct the ButtonID start value for radio buttons

                return result;
            }
            else
            {
                // [OPTION 2] Show Emulated Form
                frmTaskDialog td = new frmTaskDialog();
                td.Title = Title;
                td.MainInstruction = MainInstruction;
                td.Content = Content;
                td.ExpandedInfo = ExpandedInfo;
                td.Footer = Footer;
                td.RadioButtons = RadioButtons;
                td.CommandButtons = CommandButtons;
                td.Buttons = Buttons;
                td.MainIcon = MainIcon;
                td.CustomMainIcon = customIcon;
                td.FooterIcon = FooterIcon;
                td.VerificationText = VerificationText;
                td.Width = EmulatedFormWidth;
                td.BuildForm();
                DialogResult result = td.ShowDialog(owner);
                return new TaskDialogResult(td.CommandButtonClickedIndex, td.RadioButtonIndex,td.VerificationCheckBoxChecked, result);
            }
        }
        #endregion

        //--------------------------------------------------------------------------------
        #region MessageBox
        //--------------------------------------------------------------------------------
        static public TaskDialogResult ShowTaskDialogBox(IWin32Window owner, string Title,
                                              string MainInstruction,
                                              string Content,
                                              string ExpandedInfo,
                                              string Footer,
                                              string VerificationText,
                                              eTaskDialogButtons Buttons,
                                              eSysIcons MainIcon,
                                              eSysIcons FooterIcon)
        {
            return ShowTaskDialogBox(owner, Title, MainInstruction, Content, ExpandedInfo, Footer, VerificationText,
                                     "", "", Buttons, MainIcon, FooterIcon, null);
        }

        static public DialogResult ShowOptionalMessageBox(IWin32Window owner, string settingsKey, string Title,
                                      string MainInstruction,
                                      string Content,
                                      eTaskDialogButtons Buttons,
                                      eSysIcons MainIcon,
                                      DialogResult forceSavedAnswer)
        {
            bool shouldShow = Conversion.ToBoolean(Settings.GetValue("MessageBoxes", settingsKey, true));
            if (shouldShow)
            {
                string verificationText = (Buttons == eTaskDialogButtons.YesNo || Buttons == eTaskDialogButtons.YesNoCancel) ? BaseForm.RM.GetString("&Never ask again") : BaseForm.RM.GetString("&Never tell again");

                TaskDialogResult result = ShowTaskDialogBox(owner, Title, MainInstruction, Content, string.Empty, "", verificationText, "", "", Buttons, MainIcon, 0, null);

                if (result.DialogResult != DialogResult.Cancel)
                {
                    Settings.SetValue("MessageBoxes", settingsKey, !result.VerificationChecked);
                    Settings.SetValue("MessageBoxes", "Result" + settingsKey, result.DialogResult);
                }

                return result.DialogResult;
            }
            else
            {
                if (Buttons == eTaskDialogButtons.YesNo || Buttons == eTaskDialogButtons.YesNoCancel)
                {
                    // For some dialogs, only one saved answer makes sense. Example: Do you want to erase the disc?
                    if (forceSavedAnswer != DialogResult.None)
                    {
                        return forceSavedAnswer;
                    }

                    // Return Yes or No?
                    DialogResult res = (DialogResult) Settings.GetValue("MessageBoxes", "Result" + settingsKey, DialogResult.No);
                    return (res == DialogResult.Yes) ? DialogResult.Yes : DialogResult.No;
                }
                else
                {
                    return DialogResult.OK;
                }
            }
        }

        //--------------------------------------------------------------------------------
        static public TaskDialogResult ShowTaskDialogBox(IWin32Window owner, string Title,
                                              string MainInstruction,
                                              string Content,
                                              eTaskDialogButtons Buttons,
                                              eSysIcons MainIcon)
        {
            return ShowTaskDialogBox(owner, Title, MainInstruction, Content, "", "", "", Buttons, MainIcon, eSysIcons.Information);
        }

        //--------------------------------------------------------------------------------
        #endregion

        //--------------------------------------------------------------------------------
        #region ShowRadioBox
        //--------------------------------------------------------------------------------
        static public TaskDialogResult ShowRadioBox(IWin32Window owner, string Title,
                                       string MainInstruction,
                                       string Content,
                                       string ExpandedInfo,
                                       string Footer,
                                       string VerificationText,
                                       string RadioButtons,
                                       eSysIcons MainIcon,
                                       eSysIcons FooterIcon)
        {
            TaskDialogResult res = ShowTaskDialogBox(owner, Title, MainInstruction, Content, ExpandedInfo, Footer, VerificationText,
                                                 RadioButtons, "", eTaskDialogButtons.OKCancel,
                                                 MainIcon, FooterIcon, null);
            return res;
        }

        //--------------------------------------------------------------------------------
        // Simple overloaded version
        static public TaskDialogResult ShowRadioBox(IWin32Window owner, string Title,
                                       string MainInstruction,
                                       string Content,
                                       string RadioButtons)
        {
            return ShowRadioBox(owner, Title, MainInstruction, Content, "", "", "", RadioButtons, eSysIcons.Question, eSysIcons.Information);
        }
        #endregion

        //--------------------------------------------------------------------------------
        #region ShowCommandBox
        //--------------------------------------------------------------------------------
        static public TaskDialogResult ShowCommandBox(IWin32Window owner, string Title,
                                         string MainInstruction,
                                         string Content,
                                         string ExpandedInfo,
                                         string Footer,
                                         string VerificationText,
                                         string CommandButtons,
                                         bool ShowCancelButton,
                                         eSysIcons MainIcon,
                                         eSysIcons FooterIcon)
        {
            TaskDialogResult res = ShowTaskDialogBox(owner, Title, MainInstruction, Content, ExpandedInfo, Footer, VerificationText,
                                                 "", CommandButtons, (ShowCancelButton ? eTaskDialogButtons.Cancel : eTaskDialogButtons.None),
                                                 MainIcon, FooterIcon, null);
            return res;
        }

        //--------------------------------------------------------------------------------
        // Simple overloaded version
        static public TaskDialogResult ShowCommandBox(IWin32Window owner, string Title, string MainInstruction, string Content, string CommandButtons, bool ShowCancelButton)
        {
            return ShowCommandBox(owner, Title, MainInstruction, Content, "", "", "", CommandButtons, ShowCancelButton,
                                  eSysIcons.Question, eSysIcons.Information);
        }

        #endregion

        //--------------------------------------------------------------------------------
    }
}
