﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using CDBurnerXP.IO;

namespace CDBurnerXP.Net.Updater
{
    static class Program
    {
        /// <summary>
        /// Entry point
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            Application.SetCompatibleTextRenderingDefault(false);
            
            try
            {
                CheckForUpdateAndInstall();
            }
            catch (Exception ex)
            {
                MessageBox.Show("An error occured when trying to execute the automated update: " + ex.Message, "Automatic update", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            finally
            {
                // Restart CDBurnerXP in any case
                // Important: Keep original command line arguments
                List<string> originalArguments = new List<string>(args);
                originalArguments.Add("/UpdateDone");

                Process.Start("cdbxpp.exe", String.Join(" ", AutoUpdate.EscapeShellArgument(originalArguments.ToArray())));
            }
        }

        /// <summary>
        /// Checks for ready-to-install updates and installs them.
        /// </summary>
        private static void CheckForUpdateAndInstall()
        {
            string updateFile = AutoUpdate.GetLatestUpdate();
            if (!string.IsNullOrEmpty(updateFile))
            {
                List<string> args = new List<string>();
                // Make sure that installer works on correct directory
                args.Add("/DIR=" + AutoUpdate.EscapeShellArgument(Application.StartupPath));

#if !DEBUG
                args.Add("/SILENT");
                // Updater will always require a restart. Ignore.
                args.Add("/NORESTART");
#endif
                Process updater = Process.Start(updateFile, String.Join(" ", args.ToArray()));
                updater.WaitForExit();

#if !DEBUG
                // Only delete setup if successful.
                // --> Do not redownload if user cancels the process.
                if (updater.ExitCode == 0)
                {
                    // Clean up all old updates and the current one
                    PathEx.TryDeleteDirectory(AutoUpdate.PathToDownloadedUpdates);
                }
#endif
            }
        }
    }
}
